@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -e
 
 while getopts "q" flag
 do
@@ -17,9 +17,9 @@ done
 echo "Updating configuration..."
 echo "Running aclocal"
 if which aclocal > /dev/null 2>&1; then
-  aclocal -I src/cf
+  aclocal -I src/cf -I src/external/rra-c-util/m4
 elif which aclocal-1.10 > /dev/null 2>&1; then
-  aclocal-1.10 -I src/cf
+  aclocal-1.10 -I src/cf -I src/external/rra-c-util/m4
 else
   echo "No aclocal found on your system (looked for aclocal & aclocal-1.10)"
   exit 1
@@ -28,7 +28,7 @@ fi
 echo "Running autoconf"
 autoconf
 echo "Running autoconf for configure-libafs"
-autoconf configure-libafs.in > configure-libafs
+autoconf configure-libafs.ac > configure-libafs
 chmod +x configure-libafs
 echo "Running autoheader"
 autoheader
@@ -36,7 +36,7 @@ autoheader
 #automake
 
 echo "Deleting autom4te.cache directory"
-rm -r autom4te.cache
+rm -rf autom4te.cache
 
 if [ $skipman ] ; then
     echo "Skipping man page build"
@@ -45,6 +45,7 @@ else
     # pod2man available.
     if test -d doc/man-pages ; then
         echo "Building man pages"
+        perl doc/man-pages/merge-pod doc/man-pages/pod*/*.in
         (cd doc/man-pages && ./generate-man)
     fi
 fi
