@@ -47,7 +47,9 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include "cm_volstat.h"
 #include "cm_volume.h"
 #include "cm_dcache.h"
+#include "cm_direct.h"
 #include "cm_access.h"
+#include "cm_eacces.h"
 #include "cm_dir.h"
 #include "cm_utils.h"
 #include "cm_vnodeops.h"
@@ -61,6 +63,7 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include "cm_freelance.h"
 #include "cm_performance.h"
 #include "cm_rdr.h"
+#include "rawops.h"
 #include "afsd_init.h"
 #include "afsd_eventlog.h"
 
@@ -113,6 +116,9 @@ extern int cm_fakeGettingCallback;			// 1 if currently updating the fake root.af
 
 extern int cm_dnsEnabled;
 extern int cm_readonlyVolumeVersioning;
+extern int cm_shortNames;
+extern int cm_directIO;
+extern int cm_volumeInfoReadOnlyFlag;
 
 extern long rx_mtu;
 
@@ -124,6 +130,8 @@ extern afs_uint32 smb_Enabled;
 
 extern int cm_virtualCache;
 
+extern afs_int32 cm_verifyData;
+
 #define DFS_SUPPORT 1
 #define LOG_PACKET 1
 #undef  NOTSERVICE
