@@ -108,7 +108,7 @@
 #define AFS_EXTENTS_RESULT_TAG       'XEFA'
 #define AFS_SYS_NAME_NODE_TAG        'NSFA'
 #define AFS_REPARSE_NAME_TAG         'NRFA'
-#define AFS_NAME_ARRAY_TAG           'ANFA'
+#define AFS_NAME_ARRAY_TAG           'RAFA'
 #define AFS_OBJECT_INFO_TAG          'IOFA'
 #define AFS_NP_OBJECT_INFO_TAG       'ONFA'
 #define AFS_DIR_SNAPSHOT_TAG         'SSFA'
@@ -128,6 +128,7 @@
 #define AFS_PROCESS_AG_CB_TAG        'APFA'
 #define AFS_BYTERANGE_TAG            '_RBA'
 #define AFS_HARDLINK_REQUEST_TAG     'LFFA'
+#define AFS_SYMLINK_REQUEST_TAG      'YSFA'
 
 #define __Enter
 
@@ -158,12 +159,13 @@
 // Debug information
 //
 
-#define AFS_DBG_FLAG_BREAK_ON_ENTRY     0x00000001   // Only enabled in checked build
-#define AFS_DBG_TRACE_TO_DEBUGGER       0x00000002
-#define AFS_DBG_FLAG_ENABLE_FORCE_CRASH 0x00000004   // Only enabled in checked build
-#define AFS_DBG_BUGCHECK_EXCEPTION      0x00000008
-#define AFS_DBG_CLEAN_SHUTDOWN          0x00000010
-#define AFS_DBG_REQUIRE_CLEAN_SHUTDOWN  0x00000020
+#define AFS_DBG_FLAG_BREAK_ON_ENTRY      0x00000001   // Only enabled in checked build
+#define AFS_DBG_TRACE_TO_DEBUGGER        0x00000002
+#define AFS_DBG_FLAG_ENABLE_FORCE_CRASH  0x00000004   // Only enabled in checked build
+#define AFS_DBG_BUGCHECK_EXCEPTION       0x00000008
+#define AFS_DBG_CLEAN_SHUTDOWN           0x00000010
+#define AFS_DBG_REQUIRE_CLEAN_SHUTDOWN   0x00000020
+#define AFS_DBG_DISABLE_SYSTEM_SID_CHECK 0x00000040
 
 //
 // Pool state
@@ -231,6 +233,11 @@ static inline void AFSBreakPoint() {
 
 #endif
 
+#define AFSDbgTrace(X) \
+{ \
+    PAFSDbgLogMsg pTraceFnc = AFSDebugTraceFnc; \
+    if( pTraceFnc != NULL) { pTraceFnc X;} \
+}
 //
 // Library control device name
 //
@@ -246,6 +253,7 @@ static inline void AFSBreakPoint() {
 #define AFS_DEVICE_FLAG_HIDE_DOT_NAMES          0x00000001
 #define AFS_DEVICE_FLAG_REDIRECTOR_SHUTDOWN     0x00000002
 #define AFS_DEVICE_FLAG_DISABLE_SHORTNAMES      0x00000004
+#define AFS_DEVICE_FLAG_DIRECT_SERVICE_IO       0x00000008
 
 #endif
 
