@@ -163,13 +163,16 @@ typedef struct _AFS_NONPAGED_FCB
     USHORT          Type;
 
     //
-    // Ranking - File Resource first, then Paging Resource
+    // Ranking - File Resource, Paging Resource,
+    //           then SectionObject Resource
     //
 
     ERESOURCE       Resource;
 
     ERESOURCE       PagingResource;
 
+    ERESOURCE       SectionObjectResource;
+
     //
     // The section object pointer
     //
@@ -376,12 +379,6 @@ typedef struct AFS_FCB
 
             LONG                ExtentCount;
 
-            //
-            // The Lazy writer thread
-            //
-
-            PETHREAD            LazyWriterThread;
-
             //
             // Current count of queued flush items for the file
             //
@@ -564,6 +561,32 @@ typedef struct _AFS_DEVICE_EXTENSION
 
             KEVENT              MemoryAvailableEvent;
 
+            //
+            // Worker Pool Queues
+            //
+
+            ERESOURCE           QueueLock;
+
+            struct _AFS_WORK_ITEM     *QueueHead;
+
+            struct _AFS_WORK_ITEM     *QueueTail;
+
+            KEVENT              WorkerQueueHasItems;
+
+            LONG                QueueItemCount;
+
+            ERESOURCE           IOQueueLock;
+
+            struct _AFS_WORK_ITEM     *IOQueueHead;
+
+            struct _AFS_WORK_ITEM     *IOQueueTail;
+
+            KEVENT              IOWorkerQueueHasItems;
+
+            LONG                IOQueueItemCount;
+
+
+
         } Control;
 
         struct
@@ -670,16 +693,6 @@ typedef struct _AFS_DEVICE_EXTENSION
 
             struct _AFS_WORKER_QUEUE_HDR *PoolHead;
 
-            ERESOURCE        QueueLock;
-
-            struct _AFS_WORK_ITEM     *QueueHead;
-
-            struct _AFS_WORK_ITEM     *QueueTail;
-
-            KEVENT           WorkerQueueHasItems;
-
-            LONG             QueueItemCount;
-
             //
             // IO Worker queue
             //
@@ -688,16 +701,6 @@ typedef struct _AFS_DEVICE_EXTENSION
 
             struct _AFS_WORKER_QUEUE_HDR *IOPoolHead;
 
-            ERESOURCE        IOQueueLock;
-
-            struct _AFS_WORK_ITEM     *IOQueueHead;
-
-            struct _AFS_WORK_ITEM     *IOQueueTail;
-
-            KEVENT           IOWorkerQueueHasItems;
-
-            LONG             IOQueueItemCount;
-
         } Library;
 
     } Specific;
@@ -749,6 +752,8 @@ NTSTATUS
                        IN GUID *AuthGroup,
                        IN PUNICODE_STRING FileName,
                        IN AFSFileID *FileId,
+                       IN WCHAR * Cell,
+                       IN ULONG   CellLength,
                        IN void  *Data,
                        IN ULONG DataLength,
                        IN OUT void *ResultBuffer,
@@ -792,6 +797,8 @@ typedef struct _AFS_LIBRARY_INIT_CB
 
     UNICODE_STRING      AFSServerName;
 
+    UNICODE_STRING      AFSMountRootName;
+
     ULONG               AFSDebugFlags;
 
     AFSFileID           GlobalRootFid;
@@ -822,6 +829,12 @@ typedef struct _AFS_LIBRARY_INIT_CB
 
 } AFSLibraryInitCB;
 
+typedef struct _AFS_TRACE_CONFIG_CB
+{
+
+    PAFSDbgLogMsg       AFSDbgLogMsg;
+
+} AFSDebugTraceConfigCB;
 
 #endif
 
