@@ -59,7 +59,7 @@ typedef struct _AFS_FILE_ID
 typedef struct _AFS_COMM_REQUEST_BLOCK
 {
 
-    AFSFileID       FileId;             /* Initialize unused elements to 0 */
+    AFSFileID       FileId;          /* Initialize unused elements to 0 */
 
     ULONG           RequestType;
 
@@ -67,17 +67,17 @@ typedef struct _AFS_COMM_REQUEST_BLOCK
 
     ULONG           RequestFlags;    /* AFS_REQUEST_FLAG_xxxx */
 
-    ULONG           NameLength;      // Length of the name in bytes
+    ULONG           NameLength;      /* Length of the name in bytes */
 
-    ULONG           DataOffset;       // This offset is from the end of the structure, including the name
+    ULONG           DataOffset;      /* This offset is from the end of the structure, including the name */
 
     ULONG           DataLength;
 
-    GUID            AuthGroup;      // Length: sizeof(GUID) */
+    GUID            AuthGroup;       /* Length: sizeof(GUID) */
 
     ULONG           ResultBufferLength;    /* Do not exceed this length in response */
 
-    LONG            QueueCount;     // Current outstanding requests in the queue
+    LONG            QueueCount;      /* Current outstanding requests in the queue */
 
     WCHAR           Name[ 1];
 
@@ -95,9 +95,9 @@ typedef struct _AFS_COMM_RESULT_BLOCK
 
     ULONG           ResultStatus;        /* NTSTATUS_xxx */
 
-    ULONG           ResultBufferLength;    /* Not to exceed AFSCommRequest ResultBufferLength */
+    ULONG           ResultBufferLength;  /* Not to exceed AFSCommRequest ResultBufferLength */
 
-    ULONG           Reserved;           /* To ease Quad Alignment */
+    ULONG           Authenticated;       /* Tokens or No? */
 
     char            ResultData[ 1];
 
@@ -108,8 +108,9 @@ typedef struct _AFS_COMM_RESULT_BLOCK
 //
 
 #define AFS_REDIR_INIT_FLAG_HIDE_DOT_FILES          0x00000001
-
 #define AFS_REDIR_INIT_NO_PAGING_FILE               0x00000002
+#define AFS_REDIR_INIT_FLAG_DISABLE_SHORTNAMES	    0x00000004
+#define AFS_REDIR_INIT_PERFORM_SERVICE_IO           0x00000008
 
 typedef struct _AFS_REDIR_INIT_INFO_CB
 {
@@ -241,9 +242,9 @@ typedef struct _AFS_DIR_ENUM_RESP
 typedef struct _AFS_VOLUME_INFORMATION
 {
 
-    LARGE_INTEGER   TotalAllocationUnits;       /* Partition Max Blocks */
+    LARGE_INTEGER   TotalAllocationUnits;       /* Volume Max Blocks (Partition or Quota) */
 
-    LARGE_INTEGER   AvailableAllocationUnits;   /* Partition Blocks Avail */
+    LARGE_INTEGER   AvailableAllocationUnits;   /* Volume Blocks Avail (Partition or Quota) */
 
     LARGE_INTEGER   VolumeCreationTime;         /* AFS Last Update - Not Creation */
 
@@ -264,10 +265,33 @@ typedef struct _AFS_VOLUME_INFORMATION
 
     ULONG           VolumeLabelLength;
 
-    WCHAR           VolumeLabel[20];            /* Volume:Cell */
+    WCHAR           VolumeLabel[128];           /* Volume */
+
+    ULONG           CellLength;
+
+    WCHAR           Cell[128];                  /* Cell */
 
 } AFSVolumeInfoCB;
 
+
+//
+// Volume size information CB passed used to satisfy
+// FileFsFullSizeInformation and FileFsSizeInformation
+//
+
+typedef struct _AFS_VOLUME_SIZE_INFORMATION
+{
+
+    LARGE_INTEGER   TotalAllocationUnits;       /* Max Blocks (Quota or Partition) */
+
+    LARGE_INTEGER   AvailableAllocationUnits;   /* Blocks Avail (Quota or Partition) */
+
+    ULONG           SectorsPerAllocationUnit;   /* = 1 */
+
+    ULONG           BytesPerSector;             /* = 1024 */
+
+} AFSVolumeSizeInfoCB;
+
 //
 // File create CB
 //
@@ -583,6 +607,66 @@ typedef struct _AFS_FILE_RENAME_RESULT_CB
 } AFSFileRenameResultCB;
 
 
+//
+// File Hard Link CB
+//
+
+typedef struct _AFS_FILE_HARDLINK_CB
+{
+
+    AFSFileID       SourceParentId;        /* Must be directory */
+
+    AFSFileID       TargetParentId;        /* Must be directory */
+
+    BOOLEAN         bReplaceIfExists;
+
+                                           /* Source Name and FileID in Common Request Block */
+
+    USHORT          TargetNameLength;
+
+    WCHAR           TargetName[ 1];
+
+} AFSFileHardLinkCB;
+
+typedef struct _AFS_FILE_HARDLINK_RESULT_CB
+{
+
+    LARGE_INTEGER   SourceParentDataVersion;
+
+    LARGE_INTEGER   TargetParentDataVersion;
+
+    AFSDirEnumEntry DirEnum;
+
+} AFSFileHardLinkResultCB;
+
+
+//
+// Create Symlink CB
+//
+
+typedef struct _AFS_CREATE_SYMLINK_CB
+{
+
+    AFSFileID       ParentId;              /* Must be directory */
+
+                                           /* Source Name and FileID in Common Request Block */
+
+    USHORT          TargetNameLength;
+
+    WCHAR           TargetName[ 1];
+
+} AFSCreateSymlinkCB;
+
+typedef struct _AFS_CREATE_SYMLINK_RESULT_CB
+{
+
+    LARGE_INTEGER   ParentDataVersion;
+
+    AFSDirEnumEntry DirEnum;
+
+} AFSCreateSymlinkResultCB;
+
+
 //
 // Control structures for AFS_REQUEST_TYPE_EVAL_TARGET_BY_ID
 // and AFS_REQUEST_TYPE_EVAL_TARGET_BY_NAME
@@ -1187,5 +1271,45 @@ typedef struct _AFS_REPARSE_TAG_INFORMATION
 
 } AFSReparseTagInfo;
 
+//
+// File IO CB
+//
+
+typedef struct _AFS_FILE_IO_CB
+{
+
+    LARGE_INTEGER   IOOffset;
+
+    ULONG           IOLength;
+
+    void           *MappedIOBuffer; // Mapped buffer for access by service
+
+    void           *SystemIOBuffer; // Service should not access this pointer
+
+    void           *SystemIOBufferMdl; // The service should not access this pointer
+
+    LARGE_INTEGER   LastAccessTime;
+
+    LARGE_INTEGER   LastWriteTime;
+
+    LARGE_INTEGER   ChangeTime;
+
+    LARGE_INTEGER   CreateTime;
+
+    LARGE_INTEGER   EndOfFile;
+
+} AFSFileIOCB;
+
+typedef struct _AFS_FILE_IO_RESULT_CB
+{
+
+    ULONG           Length;
+
+    LARGE_INTEGER   DataVersion;
+
+    LARGE_INTEGER   Expiration;		/* FILETIME */
+
+} AFSFileIOResultCB;
+
 #endif /* _AFS_USER_STRUCT_H */
 
