@@ -55,6 +55,8 @@ AFSReleaseFid( IN AFSFileID *FileId)
                                       NULL,
                                       0,
                                       NULL,
+                                      0,
+                                      NULL,
                                       NULL);
     }
 
@@ -67,6 +69,8 @@ AFSProcessRequest( IN ULONG RequestType,
                    IN GUID *AuthGroup,
                    IN PUNICODE_STRING FileName,
                    IN AFSFileID *FileId,
+                   IN WCHAR *Cell,
+                   IN ULONG  CellLength,
                    IN void  *Data,
                    IN ULONG DataLength,
                    IN OUT void *ResultBuffer,
@@ -104,11 +108,11 @@ AFSProcessRequest( IN ULONG RequestType,
         // Grab the pool resource and check the state
         //
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSProcessRequest Acquiring IrpPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSProcessRequest Acquiring IrpPoolLock lock %p EXCL %08lX\n",
                       &pCommSrvc->IrpPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pCommSrvc->IrpPoolLock,
                         TRUE);
@@ -261,10 +265,10 @@ AFSProcessRequest( IN ULONG RequestType,
 
         if( AFSIsNoPAGAuthGroup( &pPoolEntry->AuthGroup))
         {
-            AFSDbgLogMsg( 0,
+            AFSDbgTrace(( 0,
                           0,
                           "AFSProcessRequest NoPAG Auth Group %08lX\n",
-                          PsGetCurrentThread());
+                          PsGetCurrentThread()));
         }
 
         //
@@ -356,9 +360,11 @@ try_exit:
                         FALSE);
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()))
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()))
     {
 
+        AFSDumpTraceFilesFnc();
+
         if( bReleasePool)
         {
 
@@ -383,6 +389,158 @@ try_exit:
     return ntStatus;
 }
 
+static
+NTSTATUS
+AFSCheckIoctlPermissions( IN ULONG ControlCode)
+{
+    switch ( ControlCode)
+    {
+        //
+        // First the FS ioctls
+        //
+
+        case IOCTL_AFS_INITIALIZE_CONTROL_DEVICE:
+
+            //
+            // Only a System Service can run this (unless we are compiled
+            // for debug with the correct flags set.
+            //
+
+            if ( !AFSIsUser( SeExports->SeLocalSystemSid)
+#if DBG
+                && !BooleanFlagOn( AFSDebugFlags, AFS_DBG_DISABLE_SYSTEM_SID_CHECK)
+#endif
+                )
+            {
+
+                return STATUS_ACCESS_DENIED;
+            }
+            return STATUS_SUCCESS;
+
+        case IOCTL_AFS_INITIALIZE_REDIRECTOR_DEVICE:
+        case IOCTL_AFS_PROCESS_IRP_REQUEST:
+        case IOCTL_AFS_PROCESS_IRP_RESULT:
+        case IOCTL_AFS_SYSNAME_NOTIFICATION:
+        case IOCTL_AFS_SHUTDOWN:
+
+            //
+            // Once initialized, only the service can call these
+            //
+
+            if ( !AFSIsService())
+            {
+
+                return STATUS_ACCESS_DENIED;
+            }
+            return STATUS_SUCCESS;
+
+        case IOCTL_AFS_CONFIGURE_DEBUG_TRACE:
+        case IOCTL_AFS_GET_TRACE_BUFFER:
+        case IOCTL_AFS_FORCE_CRASH:
+
+            //
+            // Any admin can call these
+            //
+
+            if ( !AFSIsInGroup( SeExports->SeAliasAdminsSid))
+            {
+
+                return STATUS_ACCESS_DENIED;
+            }
+            return STATUS_SUCCESS;
+
+        case IOCTL_AFS_AUTHGROUP_CREATE_AND_SET:
+        case IOCTL_AFS_AUTHGROUP_QUERY:
+        case IOCTL_AFS_AUTHGROUP_SET:
+        case IOCTL_AFS_AUTHGROUP_RESET:
+        case IOCTL_AFS_AUTHGROUP_LOGON_CREATE:
+        case IOCTL_AFS_AUTHGROUP_SID_CREATE:
+        case IOCTL_AFS_AUTHGROUP_SID_QUERY:
+
+            //
+            // Anyone can call these.
+            //
+
+            return STATUS_SUCCESS;
+
+        //
+        // And now the LIB ioctls
+        //
+
+        case IOCTL_AFS_INITIALIZE_LIBRARY_DEVICE:
+        case IOCTL_AFS_CONFIG_LIBRARY_TRACE:
+
+            //
+            // Only the kernel can issue this
+            //
+
+            return STATUS_ACCESS_DENIED;
+
+        case IOCTL_AFS_STATUS_REQUEST:
+        case 0x140390:      // IOCTL_LMR_DISABLE_LOCAL_BUFFERING
+
+            //
+            // Anyone can call these.
+            //
+
+            return STATUS_SUCCESS;
+
+        case IOCTL_AFS_ADD_CONNECTION:
+        case IOCTL_AFS_CANCEL_CONNECTION:
+        case IOCTL_AFS_GET_CONNECTION:
+        case IOCTL_AFS_LIST_CONNECTIONS:
+        case IOCTL_AFS_GET_CONNECTION_INFORMATION:
+
+            //
+            // These must only be called by the network provider but we
+            // don't have a method of enforcing that at the moment.
+            //
+
+            return STATUS_SUCCESS;
+
+        case IOCTL_AFS_SET_FILE_EXTENTS:
+        case IOCTL_AFS_RELEASE_FILE_EXTENTS:
+        case IOCTL_AFS_SET_FILE_EXTENT_FAILURE:
+        case IOCTL_AFS_INVALIDATE_CACHE:
+        case IOCTL_AFS_NETWORK_STATUS:
+        case IOCTL_AFS_VOLUME_STATUS:
+
+            //
+            // Again, service only
+            //
+
+            if ( !AFSIsService())
+            {
+
+                return STATUS_ACCESS_DENIED;
+            }
+            return STATUS_SUCCESS;
+
+        case IOCTL_AFS_GET_OBJECT_INFORMATION:
+
+            //
+            // Admins only
+            //
+
+            if ( !AFSIsInGroup( SeExports->SeAliasAdminsSid))
+            {
+
+                return STATUS_ACCESS_DENIED;
+            }
+            return STATUS_SUCCESS;
+
+        default:
+
+            //
+            // NOTE that for security we police all known functions here
+            // and return STATUS_NOT_IMPLEMENTED.  So new ioctls need to
+            // be added both here and either below or in
+            // ..\lib\AFSDevControl.cpp
+            //
+
+            return STATUS_NOT_IMPLEMENTED;
+    }
+}
 NTSTATUS
 AFSProcessControlRequest( IN PIRP Irp)
 {
@@ -392,7 +550,6 @@ AFSProcessControlRequest( IN PIRP Irp)
     ULONG               ulIoControlCode;
     BOOLEAN             bCompleteRequest = TRUE;
     AFSDeviceExt       *pDevExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
-    ULONG               ulBytesProcessed = 0;
 
     __try
     {
@@ -401,16 +558,21 @@ AFSProcessControlRequest( IN PIRP Irp)
 
         ulIoControlCode = pIrpSp->Parameters.DeviceIoControl.IoControlCode;
 
+        ntStatus = AFSCheckIoctlPermissions( ulIoControlCode);
+
+        if ( !NT_SUCCESS( ntStatus))
+        {
+            try_return( ntStatus);
+        }
+
         switch( ulIoControlCode)
         {
 
             case IOCTL_AFS_INITIALIZE_CONTROL_DEVICE:
             {
-
                 //
                 // Go intialize the pool
                 //
-
                 ntStatus = AFSInitIrpPool();
 
                 if( !NT_SUCCESS( ntStatus))
@@ -429,6 +591,8 @@ AFSProcessControlRequest( IN PIRP Irp)
 
                 pIrpSp->FileObject->FsContext = (void *)((ULONG_PTR)pIrpSp->FileObject->FsContext | AFS_CONTROL_INSTANCE);
 
+                AFSRegisterService();
+
                 break;
             }
 
@@ -633,7 +797,6 @@ AFSProcessControlRequest( IN PIRP Irp)
             case IOCTL_AFS_AUTHGROUP_CREATE_AND_SET:
             {
 
-
                 AFSAuthGroupRequestCB *pAuthGroupRequestCB = (AFSAuthGroupRequestCB *)Irp->AssociatedIrp.SystemBuffer;
 
                 if( pAuthGroupRequestCB == NULL ||
@@ -780,15 +943,17 @@ AFSProcessControlRequest( IN PIRP Irp)
             }
         }
 
-//try_exit:
-
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()))
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()))
     {
 
         ntStatus = STATUS_UNSUCCESSFUL;
+
+        AFSDumpTraceFilesFnc();
     }
 
+try_exit:
+
     if( bCompleteRequest)
     {
 
@@ -822,20 +987,20 @@ AFSInitIrpPool()
         // locks. We also do this in the tear down of the pool
         //
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInitIrpPool Acquiring IrpPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSInitIrpPool Acquiring IrpPoolLock lock %p EXCL %08lX\n",
                       &pCommSrvc->IrpPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pCommSrvc->IrpPoolLock,
                           TRUE);
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInitIrpPool Acquiring ResultPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSInitIrpPool Acquiring ResultPoolLock lock %p EXCL %08lX\n",
                       &pCommSrvc->ResultPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pCommSrvc->ResultPoolLock,
                           TRUE);
@@ -899,7 +1064,6 @@ void
 AFSCleanupIrpPool()
 {
 
-    NTSTATUS        ntStatus = STATUS_SUCCESS;
     AFSPoolEntry   *pEntry = NULL, *pNextEntry = NULL;
     AFSDeviceExt   *pDevExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
     AFSCommSrvcCB  *pCommSrvc = (AFSCommSrvcCB *)&pDevExt->Specific.Control.CommServiceCB;
@@ -912,20 +1076,20 @@ AFSCleanupIrpPool()
         // Irp pool then the result pool lock
         //
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSCleanupIrpPool Acquiring IrpPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSCleanupIrpPool Acquiring IrpPoolLock lock %p EXCL %08lX\n",
                       &pCommSrvc->IrpPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pCommSrvc->IrpPoolLock,
                         TRUE);
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSCleanupIrpPool Acquiring ResultPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSCleanupIrpPool Acquiring ResultPoolLock lock %p EXCL %08lX\n",
                       &pCommSrvc->ResultPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pCommSrvc->ResultPoolLock,
                         TRUE);
@@ -963,7 +1127,7 @@ AFSCleanupIrpPool()
                 // Here we need to complete the irp, cancelled, and delete the data block
                 //
 
-                pEntry->ResultStatus = STATUS_CANCELLED;
+                pEntry->ResultStatus = (ULONG) STATUS_CANCELLED;
 
                 KeSetEvent( &pEntry->Event,
                             0,
@@ -1009,7 +1173,22 @@ AFSCleanupIrpPool()
 
             pNextEntry = pEntry->fLink;
 
-            pEntry->ResultStatus = STATUS_CANCELLED;
+            pEntry->ResultStatus = (ULONG) STATUS_CANCELLED;
+
+            if( pEntry->RequestType == AFS_REQUEST_TYPE_PROCESS_READ_FILE ||
+                pEntry->RequestType == AFS_REQUEST_TYPE_PROCESS_WRITE_FILE)
+            {
+
+                AFSFileIOCB *pFileIO = (AFSFileIOCB *)pEntry->Data;
+
+                if( pFileIO->MappedIOBuffer != NULL)
+                {
+                    MmUnmapLockedPages( pFileIO->MappedIOBuffer,
+                                        (PMDL)pFileIO->SystemIOBufferMdl);
+
+                    pFileIO->MappedIOBuffer = NULL;
+                }
+            }
 
             //
             // Here we will set the event of the requestor and let the blocked thread
@@ -1040,6 +1219,7 @@ AFSCleanupIrpPool()
         //
 
         AFSReleaseResource( &pCommSrvc->ResultPoolLock);
+
     }
 
     return;
@@ -1055,11 +1235,11 @@ AFSInsertRequest( IN AFSCommSrvcCB *CommSrvc,
     __Enter
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInsertRequest Acquiring IrpPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSInsertRequest Acquiring IrpPoolLock lock %p EXCL %08lX\n",
                       &CommSrvc->IrpPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &CommSrvc->IrpPoolLock,
                         TRUE);
@@ -1130,11 +1310,11 @@ AFSProcessIrpRequest( IN PIRP Irp)
 
         pRequest = (AFSCommRequest *)Irp->AssociatedIrp.SystemBuffer;
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSProcessIrpRequest Acquiring IrpPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSProcessIrpRequest Acquiring IrpPoolLock lock %p EXCL %08lX\n",
                       &pCommSrvc->IrpPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pCommSrvc->IrpPoolLock,
                         TRUE);
@@ -1223,11 +1403,11 @@ AFSProcessIrpRequest( IN PIRP Irp)
             // Grab the lock on the request pool
             //
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                           AFS_TRACE_LEVEL_VERBOSE,
-                          "AFSProcessIrpRequest Acquiring IrpPoolLock (WAIT) lock %08lX EXCL %08lX\n",
+                          "AFSProcessIrpRequest Acquiring IrpPoolLock (WAIT) lock %p EXCL %08lX\n",
                           &pCommSrvc->IrpPoolLock,
-                          PsGetCurrentThread());
+                          PsGetCurrentThread()));
 
             AFSAcquireExcl( &pCommSrvc->IrpPoolLock,
                             TRUE);
@@ -1386,6 +1566,26 @@ AFSProcessIrpRequest( IN PIRP Irp)
                                    pRequest->NameLength);
                 }
 
+                //
+                // If this is an IO request then need to map the system buffer to the service process
+                //
+
+                if( pEntry->RequestType == AFS_REQUEST_TYPE_PROCESS_READ_FILE ||
+                    pEntry->RequestType == AFS_REQUEST_TYPE_PROCESS_WRITE_FILE)
+                {
+
+                    AFSFileIOCB *pFileIO = (AFSFileIOCB *)pEntry->Data;
+
+                    ASSERT( pFileIO->SystemIOBuffer != NULL);
+
+                    pFileIO->MappedIOBuffer = MmMapLockedPagesSpecifyCache( (PMDL)pFileIO->SystemIOBufferMdl,
+                                                                            UserMode,
+                                                                            MmCached,
+                                                                            NULL,
+                                                                            FALSE,
+                                                                            NormalPagePriority);
+                }
+
                 pRequest->DataOffset = 0;
 
                 pRequest->DataLength = pEntry->DataLength;
@@ -1423,11 +1623,11 @@ AFSProcessIrpRequest( IN PIRP Irp)
                     pEntry->fLink = NULL;
                     pEntry->bLink = NULL;
 
-                    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                    AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                                   AFS_TRACE_LEVEL_VERBOSE,
-                                  "AFSProcessIrpRequest Acquiring ResultPoolLock lock %08lX EXCL %08lX\n",
+                                  "AFSProcessIrpRequest Acquiring ResultPoolLock lock %p EXCL %08lX\n",
                                   &pCommSrvc->ResultPoolLock,
-                                  PsGetCurrentThread());
+                                  PsGetCurrentThread()));
 
                     AFSAcquireExcl( &pCommSrvc->ResultPoolLock,
                                     TRUE);
@@ -1477,7 +1677,6 @@ AFSProcessIrpResult( IN PIRP Irp)
 
     NTSTATUS            ntStatus = STATUS_SUCCESS;
     AFSDeviceExt       *pDevExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
-    IO_STACK_LOCATION  *pIrpSp = IoGetCurrentIrpStackLocation( Irp);
     AFSCommSrvcCB      *pCommSrvc = NULL;
     AFSPoolEntry       *pCurrentEntry = NULL;
     AFSCommResult      *pResult = NULL;
@@ -1504,11 +1703,11 @@ AFSProcessIrpResult( IN PIRP Irp)
         // Go look for our entry
         //
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSProcessIrpResult Acquiring ResultPoolLock lock %08lX EXCL %08lX\n",
+                      "AFSProcessIrpResult Acquiring ResultPoolLock lock %p EXCL %08lX\n",
                       &pCommSrvc->ResultPoolLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pCommSrvc->ResultPoolLock,
                           TRUE);
@@ -1577,6 +1776,25 @@ AFSProcessIrpResult( IN PIRP Irp)
             try_return( ntStatus = STATUS_INVALID_PARAMETER);
         }
 
+        //
+        // If this is an IO request, unmap the user buffer
+        //
+
+        if( pCurrentEntry->RequestType == AFS_REQUEST_TYPE_PROCESS_READ_FILE ||
+            pCurrentEntry->RequestType == AFS_REQUEST_TYPE_PROCESS_WRITE_FILE)
+        {
+
+            AFSFileIOCB *pFileIO = (AFSFileIOCB *)pCurrentEntry->Data;
+
+            if( pFileIO->MappedIOBuffer != NULL)
+            {
+                MmUnmapLockedPages( pFileIO->MappedIOBuffer,
+                                    (PMDL)pFileIO->SystemIOBufferMdl);
+
+                pFileIO->MappedIOBuffer = NULL;
+            }
+        }
+
         //
         // OK, move in the result information
         //
