@@ -55,6 +55,7 @@ AFSExceptionFilter( IN CHAR *FunctionString,
                     IN ULONG Code,
                     IN PEXCEPTION_POINTERS ExceptPtrs)
 {
+    UNREFERENCED_PARAMETER(Code);
 
     PEXCEPTION_RECORD ExceptRec;
     PCONTEXT Context;
@@ -66,7 +67,7 @@ AFSExceptionFilter( IN CHAR *FunctionString,
 
         Context = ExceptPtrs->ContextRecord;
 
-        AFSDbgLogMsg( 0,
+        AFSDbgTrace(( 0,
                       0,
                       "AFSExceptionFilter (Framework) - EXR %p CXR %p Function %s Code %08lX Address %p Routine %p\n",
                       ExceptRec,
@@ -74,7 +75,7 @@ AFSExceptionFilter( IN CHAR *FunctionString,
                       FunctionString,
                       ExceptRec->ExceptionCode,
                       ExceptRec->ExceptionAddress,
-                      (void *)AFSExceptionFilter);
+                      (void *)AFSExceptionFilter));
 
         DbgPrint("**** Exception Caught in AFS Redirector ****\n");
 
@@ -216,11 +217,11 @@ AFSReleaseResource( IN PERESOURCE Resource)
     if( Resource != &AFSDbgLogLock)
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSReleaseResource Releasing lock %08lX Thread %08lX\n",
+                      "AFSReleaseResource Releasing lock %p Thread %08lX\n",
                       Resource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
     }
 
     ExReleaseResourceLite( Resource);
@@ -234,11 +235,11 @@ void
 AFSConvertToShared( IN PERESOURCE Resource)
 {
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSConvertToShared Converting lock %08lX Thread %08lX\n",
+                  "AFSConvertToShared Converting lock %p Thread %08lX\n",
                   Resource,
-                  PsGetCurrentThread());
+                  PsGetCurrentThread()));
 
     ExConvertExclusiveToSharedLite( Resource);
 
@@ -609,7 +610,6 @@ AFSUpdateRegistryParameter( IN PUNICODE_STRING ValueName,
     NTSTATUS ntStatus        = STATUS_SUCCESS;
     UNICODE_STRING paramPath, uniParamKey;
     HANDLE hParameters = 0;
-    ULONG ulDisposition = 0;
     OBJECT_ATTRIBUTES stObjectAttributes;
 
     __Enter
@@ -700,7 +700,6 @@ AFSInitializeControlDevice()
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
-    AFSProcessCB *pProcessCB = NULL;
 
     __Enter
     {
@@ -809,6 +808,14 @@ AFSInitializeControlDevice()
         pDeviceExt->Specific.Control.AuthGroupTree.TreeLock = &pDeviceExt->Specific.Control.AuthGroupTreeLock;
 
         pDeviceExt->Specific.Control.AuthGroupTree.TreeHead = NULL;
+
+        //
+        // Increase the StackSize to support the extra stack frame required
+        // for use of IoCompletion routines.
+        //
+
+        AFSDeviceObject->StackSize++;
+
     }
 
     return ntStatus;
@@ -820,7 +827,6 @@ AFSRemoveControlDevice()
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
-    AFSProcessCB *pProcessCB = NULL;
 
     __Enter
     {
@@ -909,7 +915,6 @@ AFSReadServerName()
 {
 
     NTSTATUS ntStatus        = STATUS_SUCCESS;
-    ULONG Default            = 0;
     UNICODE_STRING paramPath;
     RTL_QUERY_REGISTRY_TABLE paramTable[2];
 
@@ -1001,7 +1006,6 @@ AFSReadMountRootName()
 {
 
     NTSTATUS ntStatus        = STATUS_SUCCESS;
-    ULONG Default            = 0;
     UNICODE_STRING paramPath;
     RTL_QUERY_REGISTRY_TABLE paramTable[2];
 
@@ -1101,6 +1105,7 @@ NTSTATUS
 AFSSetSysNameInformation( IN AFSSysNameNotificationCB *SysNameInfo,
                           IN ULONG SysNameInfoBufferLength)
 {
+    UNREFERENCED_PARAMETER(SysNameInfoBufferLength);
 
     NTSTATUS         ntStatus = STATUS_SUCCESS;
     AFSDeviceExt    *pControlDevExt = (AFSDeviceExt *)AFSDeviceObject->DeviceExtension;
@@ -1145,11 +1150,11 @@ AFSSetSysNameInformation( IN AFSSysNameNotificationCB *SysNameInfo,
         // Process the request
         //
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSSetSysNameInformation Acquiring SysName lock %08lX EXCL %08lX\n",
+                      "AFSSetSysNameInformation Acquiring SysName lock %p EXCL %08lX\n",
                       pSysNameLock,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( pSysNameLock,
                         TRUE);
@@ -1255,9 +1260,9 @@ NTSTATUS
 AFSDefaultDispatch( IN PDEVICE_OBJECT DeviceObject,
                     IN PIRP Irp)
 {
+    UNREFERENCED_PARAMETER(DeviceObject);
 
     NTSTATUS            ntStatus = STATUS_INVALID_DEVICE_REQUEST;
-    PIO_STACK_LOCATION  pIrpSp = IoGetCurrentIrpStackLocation( Irp);
 
     AFSCompleteRequest( Irp,
                         ntStatus);
@@ -1274,6 +1279,8 @@ AFSSendDeviceIoControl( IN DEVICE_OBJECT *TargetDeviceObject,
                         IN ULONG OutputBufferLength,
                         OUT ULONG *ResultLength)
 {
+    UNREFERENCED_PARAMETER(OutputBuffer);
+    UNREFERENCED_PARAMETER(OutputBufferLength);
 
     NTSTATUS            ntStatus = STATUS_SUCCESS;
     PIRP                pIrp = NULL;
@@ -1328,6 +1335,11 @@ AFSSendDeviceIoControl( IN DEVICE_OBJECT *TargetDeviceObject,
         // Set the completion routine.
         //
 
+        AFSDbgTrace(( AFS_SUBSYSTEM_IO_PROCESSING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "Setting AFSIrpComplete as IoCompletion Routine Irp %p\n",
+                      pIrp));
+
         IoSetCompletionRoutine( pIrp,
                                 AFSIrpComplete,
                                 &kEvent,
@@ -1402,6 +1414,8 @@ AFSIrpComplete( IN PDEVICE_OBJECT DeviceObject,
                 IN PIRP           Irp,
                 IN PVOID          Context)
 {
+    UNREFERENCED_PARAMETER(DeviceObject);
+    UNREFERENCED_PARAMETER(Irp);
 
     KEVENT *pEvent = (KEVENT *)Context;
 
@@ -1451,13 +1465,13 @@ AFSExAllocatePoolWithTag( IN POOL_TYPE  PoolType,
             if ( bTimeout || pControlDevExt == NULL)
             {
 
-                AFSDbgLogMsg( 0,
+                AFSDbgTrace(( 0,
                               0,
                               "AFSExAllocatePoolWithTag failure Type %08lX Size %08lX Tag %08lX %08lX\n",
                               PoolType,
                               NumberOfBytes,
                               Tag,
-                              PsGetCurrentThread());
+                              PsGetCurrentThread()));
 
                 switch ( Tag ) {
 
@@ -1549,12 +1563,12 @@ AFSShutdownRedirector()
     __Enter
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
                       "%s Shutting down redirector Extent count %08lX Request count %08lX\n",
                       __FUNCTION__,
                       pControlDevExt->Specific.Control.ExtentCount,
-                      pControlDevExt->Specific.Control.OutstandingServiceRequestCount);
+                      pControlDevExt->Specific.Control.OutstandingServiceRequestCount));
 
         //
         // Set the shutdown flag so the worker is more agressive in tearing down extents
@@ -1577,10 +1591,10 @@ AFSShutdownRedirector()
         if( ntStatus == STATUS_TIMEOUT)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_WARNING,
                           "AFSShutdownRedirector Failed to complete all service requests Remaining count %08lX\n",
-                          pControlDevExt->Specific.Control.OutstandingServiceRequestCount);
+                          pControlDevExt->Specific.Control.OutstandingServiceRequestCount));
 
             try_return( ntStatus = STATUS_UNSUCCESSFUL);
         }
@@ -1602,10 +1616,10 @@ AFSShutdownRedirector()
         if( ntStatus == STATUS_TIMEOUT)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_WARNING,
                           "AFSShutdownRedirector Failed to purge all extents Remaining count %08lX\n",
-                          pControlDevExt->Specific.Control.ExtentCount);
+                          pControlDevExt->Specific.Control.ExtentCount));
 
             try_return( ntStatus = STATUS_UNSUCCESSFUL);
         }
@@ -1620,13 +1634,13 @@ AFSShutdownRedirector()
 
 try_exit:
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
                       "%s Completed shut down of redirector Extent count %08lX Request count %08lX Status %08lX\n",
                       __FUNCTION__,
                       pControlDevExt->Specific.Control.ExtentCount,
                       pControlDevExt->Specific.Control.OutstandingServiceRequestCount,
-                      ntStatus);
+                      ntStatus));
     }
 
     return ntStatus;
@@ -1649,48 +1663,48 @@ AFSAcquireFcbForLazyWrite( IN PVOID Fcb,
     // Try and acquire the Fcb resource
     //
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_IO_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_IO_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSAcquireFcbForLazyWrite Acquiring Fcb %08lX\n",
-                  Fcb);
+                  "AFSAcquireFcbForLazyWrite Acquiring Fcb %p\n",
+                  Fcb));
 
     //
     // Try and grab the paging
     //
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSAcquireFcbForLazyWrite Attempt to acquire Fcb PagingIo lock %08lX SHARED %08lX\n",
+                  "AFSAcquireFcbForLazyWrite Attempt to acquire Fcb PagingIo lock %p SHARED %08lX\n",
                   &pFcb->NPFcb->PagingResource,
-                  PsGetCurrentThread());
+                  PsGetCurrentThread()));
 
     if( AFSAcquireShared( &pFcb->NPFcb->PagingResource,
                           Wait))
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSAcquireFcbForLazyWrite Acquired Fcb PagingIo lock %08lX SHARED %08lX\n",
+                      "AFSAcquireFcbForLazyWrite Acquired Fcb PagingIo lock %p SHARED %08lX\n",
                       &pFcb->NPFcb->PagingResource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         bReleasePaging = TRUE;
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSAcquireFcbForLazyWrite Attempt to acquire Fcb SectionObject lock %08lX SHARED %08lX\n",
+                      "AFSAcquireFcbForLazyWrite Attempt to acquire Fcb SectionObject lock %p SHARED %08lX\n",
                       &pFcb->NPFcb->SectionObjectResource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         if( AFSAcquireShared( &pFcb->NPFcb->SectionObjectResource,
                               Wait))
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                           AFS_TRACE_LEVEL_VERBOSE,
-                          "AFSAcquireFcbForLazyWrite Acquired Fcb SectionObject lock %08lX SHARED %08lX\n",
+                          "AFSAcquireFcbForLazyWrite Acquired Fcb SectionObject lock %p SHARED %08lX\n",
                           &pFcb->NPFcb->SectionObjectResource,
-                          PsGetCurrentThread());
+                          PsGetCurrentThread()));
 
             bReleaseSectionObject = TRUE;
 
@@ -1729,10 +1743,10 @@ AFSReleaseFcbFromLazyWrite( IN PVOID Fcb)
 
     AFSFcb *pFcb = (AFSFcb *)Fcb;
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_IO_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_IO_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSReleaseFcbFromLazyWrite Releasing Fcb %08lX\n",
-                  Fcb);
+                  "AFSReleaseFcbFromLazyWrite Releasing Fcb %p\n",
+                  Fcb));
 
     IoSetTopLevelIrp( NULL);
 
@@ -1751,21 +1765,21 @@ AFSAcquireFcbForReadAhead( IN PVOID Fcb,
     BOOLEAN bStatus = FALSE;
     AFSFcb *pFcb = (AFSFcb *)Fcb;
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSAcquireFcbForReadAhead Attempt to acquire Fcb SectionObject lock %08lX SHARED %08lX\n",
+                  "AFSAcquireFcbForReadAhead Attempt to acquire Fcb SectionObject lock %p SHARED %08lX\n",
                   &pFcb->NPFcb->SectionObjectResource,
-                  PsGetCurrentThread());
+                  PsGetCurrentThread()));
 
     if( AFSAcquireShared( &pFcb->NPFcb->SectionObjectResource,
                           Wait))
     {
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSAcquireFcbForReadAhead Acquired Fcb SectionObject lock %08lX SHARED %08lX\n",
+                      "AFSAcquireFcbForReadAhead Acquired Fcb SectionObject lock %p SHARED %08lX\n",
                       &pFcb->NPFcb->SectionObjectResource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         bStatus = TRUE;
 
@@ -1817,9 +1831,9 @@ AFSGetCallerSID( OUT UNICODE_STRING *SIDString, OUT BOOLEAN *pbImpersonation)
             if( hToken == NULL)
             {
 
-                AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                               AFS_TRACE_LEVEL_ERROR,
-                              "AFSGetCallerSID Failed to retrieve impersonation or primary token\n");
+                              "AFSGetCallerSID Failed to retrieve impersonation or primary token\n"));
 
                 try_return( ntStatus);
             }
@@ -1834,9 +1848,10 @@ AFSGetCallerSID( OUT UNICODE_STRING *SIDString, OUT BOOLEAN *pbImpersonation)
         if( !NT_SUCCESS( ntStatus))
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSGetCallerSID Failed to retrieve information Status %08lX\n", ntStatus);
+                          "AFSGetCallerSID Failed to retrieve information Status %08lX\n",
+                          ntStatus));
 
             try_return( ntStatus);
         }
@@ -1852,19 +1867,20 @@ AFSGetCallerSID( OUT UNICODE_STRING *SIDString, OUT BOOLEAN *pbImpersonation)
         if( !NT_SUCCESS( ntStatus))
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSGetCallerSID Failed to convert sid to string Status %08lX\n", ntStatus);
+                          "AFSGetCallerSID Failed to convert sid to string Status %08lX\n",
+                          ntStatus));
 
             try_return( ntStatus);
         }
 
         *SIDString = uniSIDString;
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING | AFS_SUBSYSTEM_AUTHGROUP_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING | AFS_SUBSYSTEM_AUTHGROUP_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE_2,
                       "AFSGetCallerSID Successfully retrieved SID %wZ\n",
-                      SIDString);
+                      SIDString));
 
         if ( bPrimaryToken == FALSE &&
              pbImpersonation)
@@ -1898,6 +1914,7 @@ try_exit:
 ULONG
 AFSGetSessionId( IN HANDLE ProcessId, OUT BOOLEAN *pbImpersonation)
 {
+    UNREFERENCED_PARAMETER(ProcessId);
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     PACCESS_TOKEN hToken = NULL;
@@ -1923,9 +1940,9 @@ AFSGetSessionId( IN HANDLE ProcessId, OUT BOOLEAN *pbImpersonation)
             if( hToken == NULL)
             {
 
-                AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                               AFS_TRACE_LEVEL_ERROR,
-                              "AFSGetSessionId Failed to retrieve impersonation or primary token\n");
+                              "AFSGetSessionId Failed to retrieve impersonation or primary token\n"));
 
                 try_return( ntStatus);
             }
@@ -1941,18 +1958,18 @@ AFSGetSessionId( IN HANDLE ProcessId, OUT BOOLEAN *pbImpersonation)
         {
             ulSessionId = (ULONG)-1;
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
                           "AFSGetSessionId Failed to retrieve session id Status %08lX\n",
-                          ntStatus);
+                          ntStatus));
 
             try_return( ntStatus);
         }
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING | AFS_SUBSYSTEM_AUTHGROUP_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING | AFS_SUBSYSTEM_AUTHGROUP_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE_2,
                       "AFSGetSessionId found %08lX\n",
-                      ulSessionId);
+                      ulSessionId));
 
         if ( bPrimaryToken == FALSE &&
              pbImpersonation)
