@@ -162,6 +162,8 @@ AFSProcessRequest( IN ULONG RequestType,
                    IN GUID *AuthGroup,
                    IN PUNICODE_STRING FileName,
                    IN AFSFileID *FileId,
+                   IN WCHAR * Cell,
+                   IN ULONG   CellLength,
                    IN void  *Data,
                    IN ULONG DataLength,
                    IN OUT void *ResultBuffer,
@@ -741,6 +743,9 @@ NTSTATUS
 AFSInitializeLibrary( IN AFSFileID *GlobalRootFid,
                       IN BOOLEAN QueueRootEnumeration);
 
+NTSTATUS
+AFSConfigLibraryDebug( void);
+
 //
 // AFSRDRSupport.cpp Prototypes
 //
@@ -829,6 +834,21 @@ AFSThreadCB *
 AFSInitializeThreadCB( IN AFSProcessCB *ProcessCB,
                        IN ULONGLONG ThreadId);
 
+BOOLEAN
+AFSIsUser( IN PSID Sid);
+
+BOOLEAN
+AFSIsInGroup(IN PSID Sid);
+
+VOID
+AFSRegisterService( void);
+
+VOID
+AFSDeregisterService( void);
+
+BOOLEAN
+AFSIsService( void);
+
 };
 
 #endif /* _AFS_COMMON_H */
