@@ -55,14 +55,10 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
 {
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSRDRDeviceObject->DeviceExtension;
     AFSFcb *pFcb = NULL;
     AFSNonPagedFcb *pNPFcb = NULL;
-    IO_STATUS_BLOCK stIoSb = {0,0};
     USHORT  usFcbLength = 0;
-    ULONGLONG   ullIndex = 0;
-    AFSDirEnumEntry *pDirEnumCB = NULL;
-    AFSObjectInfoCB *pObjectInfo = NULL, *pParentObjectInfo = NULL;
+    AFSObjectInfoCB *pObjectInfo = NULL;
     AFSVolumeCB *pVolumeCB = NULL;
 
     __Enter
@@ -70,10 +66,6 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
 
         pObjectInfo = DirEntry->ObjectInformation;
 
-        pParentObjectInfo = pObjectInfo->ParentObjectInformation;
-
-        pVolumeCB = pObjectInfo->VolumeCB;
-
         if ( pObjectInfo->Fcb != NULL)
         {
 
@@ -83,18 +75,20 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
             try_return( ntStatus = STATUS_SUCCESS);
         }
 
+        pVolumeCB = pObjectInfo->VolumeCB;
+
         //
         // Allocate the Fcb and the nonpaged portion of the Fcb.
         //
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE_2,
                       "AFSInitFcb Initializing fcb for %wZ FID %08lX-%08lX-%08lX-%08lX\n",
                       &DirEntry->NameInformation.FileName,
                       pObjectInfo->FileId.Cell,
                       pObjectInfo->FileId.Volume,
                       pObjectInfo->FileId.Vnode,
-                      pObjectInfo->FileId.Unique);
+                      pObjectInfo->FileId.Unique));
 
         usFcbLength = sizeof( AFSFcb);
 
@@ -105,9 +99,9 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
         if( pFcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitFcb Failed to allocate fcb\n");
+                          "AFSInitFcb Failed to allocate fcb\n"));
 
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -124,9 +118,9 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
         if( pNPFcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitFcb Failed to allocate non-paged fcb\n");
+                          "AFSInitFcb Failed to allocate non-paged fcb\n"));
 
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -158,23 +152,23 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
 
         ExInitializeResourceLite( &pNPFcb->CcbListLock);
 
-        pFcb->Header.Resource = &pNPFcb->Resource;
-
-        pFcb->Header.PagingIoResource = &pNPFcb->PagingResource;
-
         //
         // Grab the Fcb for processing
         //
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInitFcb Acquiring Fcb lock %08lX EXCL %08lX\n",
+                      "AFSInitFcb Acquiring Fcb lock %p EXCL %08lX\n",
                       &pNPFcb->Resource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pNPFcb->Resource,
                         TRUE);
 
+        pFcb->Header.Resource = &pNPFcb->Resource;
+
+        pFcb->Header.PagingIoResource = &pNPFcb->PagingResource;
+
         pFcb->NPFcb = pNPFcb;
 
         //
@@ -283,18 +277,18 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
         if ( InterlockedCompareExchangePointer( (PVOID *)&pObjectInfo->Fcb, pFcb, NULL) != NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_WARNING,
-                          "AFSInitFcb Raced Fcb %08lX pFcb %08lX Name %wZ\n",
+                          "AFSInitFcb Raced Fcb %p pFcb %p Name %wZ\n",
                           pObjectInfo->Fcb,
                           pFcb,
-                          &DirEntry->NameInformation.FileName);
+                          &DirEntry->NameInformation.FileName));
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                           AFS_TRACE_LEVEL_VERBOSE,
-                          "AFSInitFcb Acquiring Fcb lock %08lX EXCL %08lX\n",
+                          "AFSInitFcb Acquiring Fcb lock %p EXCL %08lX\n",
                           &pObjectInfo->Fcb->NPFcb->Resource,
-                          PsGetCurrentThread());
+                          PsGetCurrentThread()));
 
             AFSReleaseResource( &pObjectInfo->NonPagedInfo->ObjectInfoLock);
 
@@ -306,11 +300,11 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
 
         AFSReleaseResource( &pObjectInfo->NonPagedInfo->ObjectInfoLock);
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInitFcb Initialized Fcb %08lX Name %wZ\n",
+                      "AFSInitFcb Initialized Fcb %p Name %wZ\n",
                       &pObjectInfo->Fcb,
-                      &DirEntry->NameInformation.FileName);
+                      &DirEntry->NameInformation.FileName));
 
 try_exit:
 
@@ -320,10 +314,10 @@ try_exit:
             if ( !NT_SUCCESS( ntStatus))
             {
 
-                AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                               AFS_TRACE_LEVEL_ERROR,
                               "AFSInitFcb Failed to initialize fcb Status %08lX\n",
-                              ntStatus);
+                              ntStatus));
             }
 
             if( pFcb != NULL)
@@ -368,18 +362,18 @@ try_exit:
 NTSTATUS
 AFSInitVolume( IN GUID *AuthGroup,
                IN AFSFileID *RootFid,
+               IN LONG VolumeReferenceReason,
                OUT AFSVolumeCB **VolumeCB)
 {
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
-    IO_STATUS_BLOCK stIoStatus = {0,0};
     AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSRDRDeviceObject->DeviceExtension;
     AFSNonPagedVolumeCB *pNonPagedVcb = NULL;
     AFSVolumeCB *pVolumeCB = NULL;
     AFSNonPagedObjectInfoCB *pNonPagedObject = NULL;
     ULONGLONG ullIndex = 0;
     BOOLEAN bReleaseLocks = FALSE;
-    AFSVolumeInfoCB stVolumeInformation;
+    AFSVolumeInfoCB stVolumeInformation = {0};
     AFSNonPagedDirectoryCB *pNonPagedDirEntry = NULL;
     LONG lCount;
 
@@ -405,10 +399,10 @@ AFSInitVolume( IN GUID *AuthGroup,
             if( !NT_SUCCESS( ntStatus))
             {
 
-                AFSDbgLogMsg( AFS_SUBSYSTEM_LOAD_LIBRARY | AFS_SUBSYSTEM_INIT_PROCESSING,
+                AFSDbgTrace(( AFS_SUBSYSTEM_LOAD_LIBRARY | AFS_SUBSYSTEM_INIT_PROCESSING,
                               AFS_TRACE_LEVEL_ERROR,
                               "AFSInitVolume AFSRetrieveVolumeInformation(RootFid) failure %08lX\n",
-                              ntStatus);
+                              ntStatus));
 
                 try_return( ntStatus);
             }
@@ -439,13 +433,15 @@ AFSInitVolume( IN GUID *AuthGroup,
                 // So we don't lock with an invalidation call ...
                 //
 
-                lCount = InterlockedIncrement( &pVolumeCB->VolumeReferenceCount);
+                lCount = AFSVolumeIncrement( pVolumeCB,
+                                             VolumeReferenceReason);
 
-                AFSDbgLogMsg( AFS_SUBSYSTEM_VOLUME_REF_COUNTING,
+                AFSDbgTrace(( AFS_SUBSYSTEM_VOLUME_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
-                              "AFSInitVolume Increment count on volume %08lX Cnt %d\n",
+                              "AFSInitVolume Increment count on volume %p Reason %u Cnt %d\n",
                               pVolumeCB,
-                              lCount);
+                              VolumeReferenceReason,
+                              lCount));
 
                 AFSReleaseResource( pDeviceExt->Specific.RDR.VolumeTree.TreeLock);
 
@@ -473,16 +469,16 @@ AFSInitVolume( IN GUID *AuthGroup,
         // into the volume tree ...
         //
 
-        pVolumeCB = (AFSVolumeCB *)AFSExAllocatePoolWithTag( NonPagedPool,
+        pVolumeCB = (AFSVolumeCB *)AFSExAllocatePoolWithTag( PagedPool,
                                                              sizeof( AFSVolumeCB),
                                                              AFS_VCB_ALLOCATION_TAG);
 
         if( pVolumeCB == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitVolume Failed to allocate the root volume cb\n");
+                          "AFSInitVolume Failed to allocate the root volume cb\n"));
 
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -501,9 +497,9 @@ AFSInitVolume( IN GUID *AuthGroup,
         if( pNonPagedVcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitVolume Failed to allocate the root non paged volume cb\n");
+                          "AFSInitVolume Failed to allocate the root non paged volume cb\n"));
 
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -522,9 +518,9 @@ AFSInitVolume( IN GUID *AuthGroup,
         if( pNonPagedObject == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitVolume Failed to allocate the root non paged object cb\n");
+                          "AFSInitVolume Failed to allocate the root non paged object cb\n"));
 
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -546,16 +542,15 @@ AFSInitVolume( IN GUID *AuthGroup,
 
         pVolumeCB->ObjectInfoTree.TreeLock = &pNonPagedVcb->ObjectInfoTreeLock;
 
-        //
-        // Bias our reference by 1
-        //
+        lCount = AFSVolumeIncrement( pVolumeCB,
+                                     VolumeReferenceReason);
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_VOLUME_REF_COUNTING,
+        AFSDbgTrace(( AFS_SUBSYSTEM_VOLUME_REF_COUNTING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInitVolume Initializing count (2) on volume %08lX\n",
-                      pVolumeCB);
-
-        pVolumeCB->VolumeReferenceCount = 2;
+                      "AFSInitVolume Initializing volume %p Reason %u count %d\n",
+                      pVolumeCB,
+                      VolumeReferenceReason,
+                      lCount));
 
         AFSAcquireExcl( pVolumeCB->VolumeLock,
                         TRUE);
@@ -570,6 +565,11 @@ AFSInitVolume( IN GUID *AuthGroup,
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
 
+        AFSDbgTrace(( AFS_SUBSYSTEM_DIRENTRY_ALLOCATION,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSInitVolume AFS_DIR_ENTRY_TAG allocated %p\n",
+                      pVolumeCB->DirectoryCB));
+
         pNonPagedDirEntry = (AFSNonPagedDirectoryCB *)AFSExAllocatePoolWithTag( NonPagedPool,
                                                                                 sizeof( AFSNonPagedDirectoryCB),
                                                                                 AFS_DIR_ENTRY_NP_TAG);
@@ -634,6 +634,11 @@ AFSInitVolume( IN GUID *AuthGroup,
 
         pVolumeCB->DirectoryCB->ObjectInformation = &pVolumeCB->ObjectInformation;
 
+        //
+        // The ObjectInformation VolumeCB pointer does not obtain
+        // a reference count.
+        //
+
         pVolumeCB->DirectoryCB->ObjectInformation->VolumeCB = pVolumeCB;
 
         //
@@ -712,6 +717,11 @@ try_exit:
                 if( pVolumeCB->DirectoryCB != NULL)
                 {
 
+                    AFSDbgTrace(( AFS_SUBSYSTEM_DIRENTRY_ALLOCATION,
+                                  AFS_TRACE_LEVEL_VERBOSE,
+                                  "AFSInitVolume AFS_DIR_ENTRY_TAG deallocating %p\n",
+                                  pVolumeCB->DirectoryCB));
+
                     AFSExFreePoolWithTag( pVolumeCB->DirectoryCB, AFS_DIR_ENTRY_TAG);
                 }
 
@@ -749,6 +759,8 @@ AFSRemoveVolume( IN AFSVolumeCB *VolumeCB)
     __Enter
     {
 
+        ASSERT( VolumeCB->VolumeReferenceCount == 0);
+
         //
         // Remove the volume from the tree and list
         // Don't process the list information for reserved entries
@@ -817,12 +829,17 @@ AFSRemoveVolume( IN AFSVolumeCB *VolumeCB)
                 AFSReleaseResource( &VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->NonPagedInfo->ObjectInfoLock);
             }
 
-            AFSDeleteObjectInfo( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
+            AFSDeleteObjectInfo( &VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
 
             ExDeleteResourceLite( &VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
 
             AFSExFreePoolWithTag( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->NonPaged, AFS_DIR_ENTRY_NP_TAG);
 
+            AFSDbgTrace(( AFS_SUBSYSTEM_DIRENTRY_ALLOCATION,
+                          AFS_TRACE_LEVEL_VERBOSE,
+                          "AFSRemoveVolume (pioctl) AFS_DIR_ENTRY_TAG deallocating %p\n",
+                          VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB));
+
             AFSExFreePoolWithTag( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB, AFS_DIR_ENTRY_TAG);
         }
 
@@ -876,6 +893,11 @@ AFSRemoveVolume( IN AFSVolumeCB *VolumeCB)
                 AFSExFreePoolWithTag( VolumeCB->DirectoryCB->NonPaged, AFS_DIR_ENTRY_NP_TAG);
             }
 
+            AFSDbgTrace(( AFS_SUBSYSTEM_DIRENTRY_ALLOCATION,
+                          AFS_TRACE_LEVEL_VERBOSE,
+                          "AFSRemoveVolume AFS_DIR_ENTRY_TAG deallocating %p\n",
+                          VolumeCB->DirectoryCB));
+
             AFSExFreePoolWithTag( VolumeCB->DirectoryCB, AFS_DIR_ENTRY_TAG);
         }
 
@@ -885,6 +907,37 @@ AFSRemoveVolume( IN AFSVolumeCB *VolumeCB)
     return ntStatus;
 }
 
+LONG
+AFSVolumeIncrement( IN AFSVolumeCB *VolumeCB,
+                    IN LONG Reason)
+{
+
+    LONG lCount;
+
+    lCount = InterlockedIncrement( &VolumeCB->VolumeReferenceCount);
+
+    InterlockedIncrement( &VolumeCB->VolumeReferences[ Reason]);
+
+    return lCount;
+}
+
+LONG
+AFSVolumeDecrement( IN AFSVolumeCB *VolumeCB,
+                    IN LONG Reason)
+{
+
+    LONG lCount;
+
+    lCount = InterlockedDecrement( &VolumeCB->VolumeReferences[ Reason]);
+
+    ASSERT( lCount >= 0);
+
+    lCount = InterlockedDecrement( &VolumeCB->VolumeReferenceCount);
+
+    ASSERT( lCount >= 0);
+
+    return lCount;
+}
 
 //
 // Function: AFSInitRootFcb
@@ -903,11 +956,10 @@ AFSInitRootFcb( IN ULONGLONG ProcessID,
                 IN AFSVolumeCB *VolumeCB)
 {
 
+    UNREFERENCED_PARAMETER(ProcessID);
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSFcb *pFcb = NULL;
     AFSNonPagedFcb *pNPFcb = NULL;
-    IO_STATUS_BLOCK stIoStatus = {0,0};
-    AFSDeviceExt *pDeviceExt = (AFSDeviceExt *)AFSRDRDeviceObject->DeviceExtension;
 
     __Enter
     {
@@ -923,9 +975,9 @@ AFSInitRootFcb( IN ULONGLONG ProcessID,
         if( pFcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitRootFcb Failed to allocate the root fcb\n");
+                          "AFSInitRootFcb Failed to allocate the root fcb\n"));
 
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -934,6 +986,7 @@ AFSInitRootFcb( IN ULONGLONG ProcessID,
                        sizeof( AFSFcb));
 
         pFcb->Header.NodeByteSize = sizeof( AFSFcb);
+
         pFcb->Header.NodeTypeCode = AFS_ROOT_FCB;
 
         pNPFcb = (AFSNonPagedFcb *)AFSExAllocatePoolWithTag( NonPagedPool,
@@ -943,9 +996,9 @@ AFSInitRootFcb( IN ULONGLONG ProcessID,
         if( pNPFcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitRootFcb Failed to allocate the non-paged fcb\n");
+                          "AFSInitRootFcb Failed to allocate the non-paged fcb\n"));
 
             try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -954,6 +1007,7 @@ AFSInitRootFcb( IN ULONGLONG ProcessID,
                        sizeof( AFSNonPagedFcb));
 
         pNPFcb->Size = sizeof( AFSNonPagedFcb);
+
         pNPFcb->Type = AFS_NON_PAGED_FCB;
 
         //
@@ -966,19 +1020,21 @@ AFSInitRootFcb( IN ULONGLONG ProcessID,
 
         ExInitializeResourceLite( &pNPFcb->Resource);
 
-        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+        ExInitializeResourceLite( &pNPFcb->PagingResource);
+
+        ExInitializeResourceLite( &pNPFcb->SectionObjectResource);
+
+        ExInitializeResourceLite( &pNPFcb->CcbListLock);
+
+        AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInitRootFcb Acquiring Fcb lock %08lX EXCL %08lX\n",
+                      "AFSInitRootFcb Acquiring Fcb lock %p EXCL %08lX\n",
                       &pNPFcb->Resource,
-                      PsGetCurrentThread());
+                      PsGetCurrentThread()));
 
         AFSAcquireExcl( &pNPFcb->Resource,
                         TRUE);
 
-        ExInitializeResourceLite( &pNPFcb->PagingResource);
-
-        ExInitializeResourceLite( &pNPFcb->CcbListLock);
-
         pFcb->Header.Resource = &pNPFcb->Resource;
 
         pFcb->Header.PagingIoResource = &pNPFcb->PagingResource;
@@ -989,23 +1045,77 @@ AFSInitRootFcb( IN ULONGLONG ProcessID,
         // Save the root Fcb in the VolumeCB
         //
 
-        VolumeCB->ObjectInformation.Fcb = pFcb;
+        pFcb->ObjectInformation = &VolumeCB->ObjectInformation;
 
         VolumeCB->ObjectInformation.VolumeCB = VolumeCB;
 
-        VolumeCB->RootFcb = pFcb;
+        AFSAcquireShared( &VolumeCB->ObjectInformation.NonPagedInfo->ObjectInfoLock,
+                          TRUE);
+        //
+        // Swap the allocated FCB into the ObjectInformation structure if it
+        // does not already have one.
+        //
 
-        pFcb->ObjectInformation = &VolumeCB->ObjectInformation;
+        if ( InterlockedCompareExchangePointer( (PVOID *)&VolumeCB->ObjectInformation.Fcb, pFcb, NULL) != NULL)
+        {
+
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
+                          AFS_TRACE_LEVEL_WARNING,
+                          "AFSInitRootFcb Raced Fcb %p pFcb %p\n",
+                          VolumeCB->ObjectInformation.Fcb,
+                          pFcb));
+
+            AFSDbgTrace(( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                          AFS_TRACE_LEVEL_VERBOSE,
+                          "AFSInitRootFcb Acquiring Fcb lock %p EXCL %08lX\n",
+                          &VolumeCB->ObjectInformation.Fcb->NPFcb->Resource,
+                          PsGetCurrentThread()));
+
+            AFSReleaseResource( &VolumeCB->ObjectInformation.NonPagedInfo->ObjectInfoLock);
+
+            AFSAcquireExcl( &VolumeCB->ObjectInformation.Fcb->NPFcb->Resource,
+                            TRUE);
+
+            try_return( ntStatus = STATUS_REPARSE);
+        }
+
+        VolumeCB->RootFcb = VolumeCB->ObjectInformation.Fcb;
+
+        AFSReleaseResource( &VolumeCB->ObjectInformation.NonPagedInfo->ObjectInfoLock);
+
+        AFSDbgTrace(( AFS_SUBSYSTEM_FCB_REF_COUNTING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSInitRootFcb Initialized Fcb %p\n",
+                      &VolumeCB->ObjectInformation.Fcb));
 
 try_exit:
 
-        if( !NT_SUCCESS( ntStatus))
+        if( !NT_SUCCESS( ntStatus) ||
+            ntStatus == STATUS_REPARSE)
         {
 
             if( pFcb != NULL)
             {
 
-                AFSRemoveRootFcb( pFcb);
+                if( pNPFcb != NULL)
+                {
+
+                    AFSReleaseResource( &pNPFcb->Resource);
+
+                    FsRtlTeardownPerStreamContexts( &pFcb->Header);
+
+                    ExDeleteResourceLite( &pNPFcb->SectionObjectResource);
+
+                    ExDeleteResourceLite( &pNPFcb->PagingResource);
+
+                    ExDeleteResourceLite( &pNPFcb->CcbListLock);
+
+                    ExDeleteResourceLite( &pNPFcb->Resource);
+
+                    AFSExFreePoolWithTag( pNPFcb, AFS_FCB_NP_ALLOCATION_TAG);
+                }
+
+                AFSExFreePoolWithTag( pFcb, AFS_FCB_ALLOCATION_TAG);
             }
         }
     }
@@ -1018,45 +1128,67 @@ try_exit:
 //
 // Description:
 //
-//      This function performs root Fcb removal/deallocation
+//      This function performs root Fcb removal/deallocation from
+//      the provided VolumeCB object.
 //
 // Return:
 //
-//      A status is returned for the function
+//      None
 //
 
 void
-AFSRemoveRootFcb( IN AFSFcb *RootFcb)
+AFSRemoveRootFcb( IN AFSVolumeCB *VolumeCB)
 {
+    AFSFcb * pRootFcb;
 
-    AFSDirectoryCB *pCurrentDirEntry = NULL;
-    AFSVolumeCB *pVolumeCB = RootFcb->ObjectInformation->VolumeCB;
+    pRootFcb = (AFSFcb *) InterlockedCompareExchangePointer( (PVOID *)&VolumeCB->ObjectInformation.Fcb,
+                                                             NULL,
+                                                             (PVOID *)&VolumeCB->ObjectInformation.Fcb);
 
-    if( RootFcb->NPFcb != NULL)
+    if ( pRootFcb == NULL)
+    {
+
+        return;
+    }
+
+    //
+    // The Fcb has been disconnected from the ObjectInformation block.
+    // Clear it from the RootFcb convenience pointer.
+    //
+
+    VolumeCB->RootFcb = NULL;
+
+    if( pRootFcb->NPFcb != NULL)
     {
 
         //
         // Now the resource
         //
 
-        ExDeleteResourceLite( &RootFcb->NPFcb->Resource);
+        ExDeleteResourceLite( &pRootFcb->NPFcb->Resource);
 
-        ExDeleteResourceLite( &RootFcb->NPFcb->PagingResource);
+        ExDeleteResourceLite( &pRootFcb->NPFcb->PagingResource);
 
-        ExDeleteResourceLite( &RootFcb->NPFcb->CcbListLock);
+        ExDeleteResourceLite( &pRootFcb->NPFcb->SectionObjectResource);
+
+        ExDeleteResourceLite( &pRootFcb->NPFcb->CcbListLock);
+
+        FsRtlTeardownPerStreamContexts( &pRootFcb->Header);
 
         //
         // The non paged region
         //
 
-        AFSExFreePoolWithTag( RootFcb->NPFcb, AFS_FCB_NP_ALLOCATION_TAG);
+        AFSExFreePoolWithTag( pRootFcb->NPFcb, AFS_FCB_NP_ALLOCATION_TAG);
+
+        pRootFcb->NPFcb = NULL;
     }
 
     //
     // And the Fcb itself
     //
 
-    AFSExFreePoolWithTag( RootFcb, AFS_FCB_ALLOCATION_TAG);
+    AFSExFreePoolWithTag( pRootFcb, AFS_FCB_ALLOCATION_TAG);
 
     return;
 }
@@ -1087,14 +1219,16 @@ AFSRemoveFcb( IN AFSFcb **ppFcb)
         return;
     }
 
+    ASSERT( pFcb->Header.NodeTypeCode != AFS_ROOT_FCB);
+
     //
     // Uninitialize the file lock if it is a file
     //
 
-    AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+    AFSDbgTrace(( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                   AFS_TRACE_LEVEL_VERBOSE,
-                  "AFSRemoveFcb Removing Fcb %08lX\n",
-                  pFcb);
+                  "AFSRemoveFcb Removing Fcb %p\n",
+                  pFcb));
 
     if( pFcb->Header.NodeTypeCode == AFS_FILE_FCB)
     {
@@ -1150,11 +1284,15 @@ AFSRemoveFcb( IN AFSFcb **ppFcb)
 }
 
 NTSTATUS
-AFSInitCcb( IN OUT AFSCcb **Ccb)
+AFSInitCcb( IN OUT AFSCcb **Ccb,
+            IN     AFSDirectoryCB *DirectoryCB,
+            IN     ACCESS_MASK     GrantedAccess,
+            IN     ULONG           FileAccess)
 {
 
     NTSTATUS Status = STATUS_SUCCESS;
     AFSCcb *pCcb = NULL;
+    LONG lCount;
 
     __Enter
     {
@@ -1170,9 +1308,9 @@ AFSInitCcb( IN OUT AFSCcb **Ccb)
         if( pCcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitCcb Failed to allocate Ccb\n");
+                          "AFSInitCcb Failed to allocate Ccb\n"));
 
             try_return( Status = STATUS_INSUFFICIENT_RESOURCES);
         }
@@ -1191,15 +1329,31 @@ AFSInitCcb( IN OUT AFSCcb **Ccb)
         if( pCcb->NPCcb == NULL)
         {
 
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+            AFSDbgTrace(( AFS_SUBSYSTEM_FILE_PROCESSING,
                           AFS_TRACE_LEVEL_ERROR,
-                          "AFSInitCcb Failed to allocate NPCcb\n");
+                          "AFSInitCcb Failed to allocate NPCcb\n"));
 
             try_return( Status = STATUS_INSUFFICIENT_RESOURCES);
         }
 
         ExInitializeResourceLite( &pCcb->NPCcb->CcbLock);
 
+        pCcb->DirectoryCB = DirectoryCB;
+
+        lCount = InterlockedIncrement( &pCcb->DirectoryCB->DirOpenReferenceCount);
+
+        AFSDbgTrace(( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSInitCcb Increment count on %wZ DE %p Ccb %p Cnt %d\n",
+                      &pCcb->DirectoryCB->NameInformation.FileName,
+                      pCcb->DirectoryCB,
+                      pCcb,
+                      lCount));
+
+        pCcb->GrantedAccess = GrantedAccess;
+
+        pCcb->FileAccess = FileAccess;
+
         //
         // Return the Ccb
         //
@@ -1239,15 +1393,15 @@ try_exit:
 //
 // Return:
 //
-//      A status is returned for the function
+//      None
 //
 
-NTSTATUS
+void
 AFSRemoveCcb( IN AFSFcb *Fcb,
               IN AFSCcb *Ccb)
 {
 
-    NTSTATUS ntStatus = STATUS_SUCCESS;
+    LONG lCount;
 
     AFSAcquireExcl( &Ccb->NPCcb->CcbLock,
                     TRUE);
@@ -1330,6 +1484,22 @@ AFSRemoveCcb( IN AFSFcb *Fcb,
         AFSExFreePoolWithTag( Ccb->NotifyMask.Buffer, AFS_GENERIC_MEMORY_7_TAG);
     }
 
+    if ( Ccb->DirectoryCB != NULL)
+    {
+
+        lCount = InterlockedDecrement( &Ccb->DirectoryCB->DirOpenReferenceCount);
+
+        AFSDbgTrace(( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
+                      AFS_TRACE_LEVEL_VERBOSE,
+                      "AFSRemoveCcb Decrement count on %wZ DE %p Ccb %p Cnt %d\n",
+                      &Ccb->DirectoryCB->NameInformation.FileName,
+                      Ccb->DirectoryCB,
+                      Ccb,
+                      lCount));
+
+        ASSERT( lCount >= 0);
+    }
+
     AFSReleaseResource( &Ccb->NPCcb->CcbLock);
 
     //
@@ -1341,8 +1511,6 @@ AFSRemoveCcb( IN AFSFcb *Fcb,
     AFSExFreePoolWithTag( Ccb->NPCcb, AFS_CCB_NP_ALLOCATION_TAG);
 
     AFSExFreePoolWithTag( Ccb, AFS_CCB_ALLOCATION_TAG);
-
-    return ntStatus;
 }
 
 NTSTATUS
