@@ -15,14 +15,11 @@
 #define __XDR_INCLUDE__
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
-#if defined(AFS_SUN55_ENV) && !defined(AFS_NONFSTRANS)
+#if defined(AFS_SUN5_ENV) && !defined(AFS_NONFSTRANS)
 #include "rpc/types.h"
 #include "rpc/auth.h"
 #include "rpc/auth_unix.h"
 #include "rpc/auth_des.h"
-#if !defined(AFS_SUN58_ENV)
-#include "rpc/auth_kerb.h"
-#endif
 #include "sys/tiuser.h"
 #include "rpc/xdr.h"
 #include "rpc/svc.h"
@@ -223,7 +220,7 @@ acl2_to_afs_call(int which, caddr_t * args, fhandle_t ** fhpp)
 	    fhp = &sargs->fh;
 	    break;
 	}
-#if defined(AFS_SUN510_ENV) 
+#if defined(AFS_SUN510_ENV)
     case ACLPROC2_GETXATTRDIR:
 	{
 	    struct GETXATTRDIR2args *sargs = (struct GETXATTRDIR2args *)args;
@@ -310,11 +307,11 @@ afs_nfs2_dispatcher(int type, afs_int32 which, char *argp,
 void
 afs_nfs2_smallfidder(struct nfsdiropres *dr)
 {
-    register fhandle_t *fhp = (fhandle_t *) & dr->dr_fhandle;
+    fhandle_t *fhp = (fhandle_t *) & dr->dr_fhandle;
     afs_int32 addr[2];
     struct vcache *vcp;
 
-#if defined(AFS_SUN57_64BIT_ENV)
+#if defined(AFS_SUN5_64BIT_ENV)
     /* See also afs_fid() */
     memcpy((char *)addr, fhp->fh_data, SIZEOF_SMALLFID);
     addr[1] = (addr[1] >> 48) & 0xffff;
@@ -695,7 +692,7 @@ afs_acl2_access(char *args, char *xp, char *exp, char *rp, char *crp)
     return;
 }
 
-#if defined(AFS_SUN510_ENV) 
+#if defined(AFS_SUN510_ENV)
 void
 afs_acl2_getxattrdir(char *args, char *xp, char *exp, char *rp, char *crp)
 {
@@ -720,7 +717,7 @@ struct afs_nfs_disp_tbl afs_acl_disp_tbl[ACL2_NPROC] = {
     {afs_acl2_setacl},
     {afs_acl2_getattr},
     {afs_acl2_access},
-#if defined(AFS_SUN510_ENV) 
+#if defined(AFS_SUN510_ENV)
     {afs_acl2_getxattrdir}
 #endif
 };
@@ -816,11 +813,7 @@ nfs3_to_afs_call(int which, caddr_t * args, nfs_fh3 ** fhpp, nfs_fh3 ** fh2pp)
     case NFSPROC3_LOOKUP:
 	{
 	    LOOKUP3args *arg = (LOOKUP3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->what.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->what.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_ACCESS:
@@ -850,84 +843,51 @@ nfs3_to_afs_call(int which, caddr_t * args, nfs_fh3 ** fhpp, nfs_fh3 ** fh2pp)
     case NFSPROC3_CREATE:
 	{
 	    CREATE3args *arg = (CREATE3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->where.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->where.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_MKDIR:
 	{
 	    MKDIR3args *arg = (MKDIR3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->where.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->where.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_SYMLINK:
 	{
 	    SYMLINK3args *arg = (SYMLINK3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->where.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->where.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_MKNOD:
 	{
 	    MKNOD3args *arg = (MKNOD3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->where.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->where.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_REMOVE:
 	{
 	    REMOVE3args *arg = (REMOVE3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->object.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->object.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_RMDIR:
 	{
 	    RMDIR3args *arg = (RMDIR3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->object.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->object.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_RENAME:
 	{
 	    RENAME3args *arg = (RENAME3args *) args;
-#ifdef AFS_SUN58_ENV
 	    fhp1 = (nfs_fh3 *) arg->from.dirp;
 	    fhp2 = (nfs_fh3 *) arg->to.dirp;
-#else
-	    fhp1 = (nfs_fh3 *) & arg->from.dir;
-	    fhp2 = (nfs_fh3 *) & arg->to.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_LINK:
 	{
 	    LINK3args *arg = (LINK3args *) args;
 	    fhp1 = (nfs_fh3 *) & arg->file;
-#ifdef AFS_SUN58_ENV
 	    fhp2 = (nfs_fh3 *) arg->link.dirp;
-#else
-	    fhp2 = (nfs_fh3 *) & arg->link.dir;
-#endif
 	    break;
 	}
     case NFSPROC3_READDIR:
@@ -1002,7 +962,7 @@ acl3_to_afs_call(int which, caddr_t * args, nfs_fh3 ** fhpp)
 	    fhp = &sargs->fh;
 	    break;
 	}
-#if defined(AFS_SUN510_ENV) 
+#if defined(AFS_SUN510_ENV)
     case ACLPROC3_GETXATTRDIR:
 	{
 	    struct GETXATTRDIR3args *sargs = (struct GETXATTRDIR3args *)args;
@@ -1038,7 +998,7 @@ afs_nfs3_dispatcher(int type, afs_int32 which, char *argp,
 	return 2;
 
     sa = (struct sockaddr *)svc_getrpccaller(rp->rq_xprt)->buf;
-    if (sa == NULL) 
+    if (sa == NULL)
 	return;
 
     if (sa->sa_family == AF_INET)
@@ -1095,7 +1055,7 @@ afs_nfs3_smallfidder(struct nfs_fh3 *fhp, int status)
     afs_int32 addr[2];
     struct vcache *vcp;
 
-#if defined(AFS_SUN57_64BIT_ENV)
+#if defined(AFS_SUN5_64BIT_ENV)
     /* See also afs_fid() */
     memcpy((char *)addr, fhp->fh3_data, 10);
     addr[1] = (addr[1] >> 48) & 0xffff;
@@ -1598,7 +1558,7 @@ afs_acl3_setacl(char *args, char *xp, char *exp, char *rp, char *crp)
     return;
 }
 
-#if defined(AFS_SUN510_ENV) 
+#if defined(AFS_SUN510_ENV)
 void
 afs_acl3_getxattrdir(char *args, char *xp, char *exp, char *rp, char *crp)
 {
@@ -1621,7 +1581,7 @@ struct afs_nfs_disp_tbl afs_acl3_disp_tbl[ACL3_NPROC] = {
     {afs_nfs2_null},
     {afs_acl3_getacl},
     {afs_acl3_setacl},
-#if defined(AFS_SUN510_ENV) 
+#if defined(AFS_SUN510_ENV)
     {afs_acl3_getxattrdir},
 #endif
 };
