@@ -9,56 +9,44 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
+#include <afs/stds.h>
+
+#include <afs/procmgmt.h>
+#include <roken.h>
 
 #ifdef IGNORE_SOME_GCC_WARNINGS
 # pragma GCC diagnostic warning "-Wdeprecated-declarations"
 #endif
 
-#include <afs/stds.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <direct.h>
-#include <io.h>
 #include <WINNT/afsevent.h>
-#else
-#include <unistd.h>
-#include <netinet/in.h>
-#include <syslog.h>
 #endif /* AFS_NT40_ENV */
-#include <afs/cellconfig.h>
+
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <rx/rx_globals.h>
-#include "bosint.h"
-#include "bnode.h"
-#include "bosprototypes.h"
 #include <rx/rxkad.h>
 #include <rx/rxstat.h>
 #include <afs/keys.h>
 #include <afs/ktime.h>
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
-#include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <afs/audit.h>
 #include <afs/cellconfig.h>
+
 #if defined(AFS_SGI_ENV)
 #include <afs/afs_args.h>
 #endif
 
+#include "bosint.h"
+#include "bnode.h"
+#include "bosprototypes.h"
+
 #define BOZO_LWP_STACKSIZE	16000
 extern struct bnode_ops fsbnode_ops, dafsbnode_ops, ezbnode_ops, cronbnode_ops;
 
@@ -66,10 +54,6 @@ struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
 static PROCESS bozo_pid;
 const char *bozo_fileName;
 FILE *bozo_logFile;
-#ifndef AFS_NT40_ENV
-static int bozo_argc = 0;
-static char** bozo_argv = NULL;
-#endif
 
 const char *DoCore;
 int DoLogging = 0;
@@ -116,13 +100,62 @@ int
 bozo_ReBozo(void)
 {
 #ifdef AFS_NT40_ENV
-    /* exit with restart code; SCM integrator process will restart bosserver with
-       the same arguments */
-    exit(BOSEXIT_RESTART);
+    /* exit with restart code; SCM integrator process will restart bosserver */
+    int status = BOSEXIT_RESTART;
+
+    /* if noauth flag is set, pass "-noauth" to new bosserver */
+    if (afsconf_GetNoAuthFlag(bozo_confdir)) {
+	status |= BOSEXIT_NOAUTH_FLAG;
+    }
+    /* if logging is on, pass "-log" to new bosserver */
+    if (DoLogging) {
+	status |= BOSEXIT_LOGGING_FLAG;
+    }
+    /* if rxbind is set, pass "-rxbind" to new bosserver */
+    if (rxBind) {
+	status |= BOSEXIT_RXBIND_FLAG;
+    }
+    exit(status);
 #else
     /* exec new bosserver process */
+    char *argv[4];
     int i = 0;
 
+    argv[i] = (char *)AFSDIR_SERVER_BOSVR_FILEPATH;
+    i++;
+
+    /* if noauth flag is set, pass "-noauth" to new bosserver */
+    if (afsconf_GetNoAuthFlag(bozo_confdir)) {
+	argv[i] = "-noauth";
+	i++;
+    }
+    /* if logging is on, pass "-log" to new bosserver */
+    if (DoLogging) {
+	argv[i] = "-log";
+	i++;
+    }
+    /* if rxbind is set, pass "-rxbind" to new bosserver */
+    if (rxBind) {
+	argv[i] = "-rxbind";
+	i++;
+    }
+#ifndef AFS_NT40_ENV
+    /* if syslog logging is on, pass "-syslog" to new bosserver */
+    if (DoSyslog) {
+	char *arg = (char *)malloc(40);	/* enough for -syslog=# */
+	if (DoSyslogFacility != LOG_DAEMON) {
+	    snprintf(arg, 40, "-syslog=%d", DoSyslogFacility);
+	} else {
+	    strcpy(arg, "-syslog");
+	}
+	argv[i] = arg;
+	i++;
+    }
+#endif
+
+    /* null-terminate argument list */
+    argv[i] = NULL;
+
     /* close random fd's */
     for (i = 3; i < 64; i++) {
 	close(i);
@@ -130,7 +163,7 @@ bozo_ReBozo(void)
 
     unlink(AFSDIR_SERVER_BOZRXBIND_FILEPATH);
 
-    execv(bozo_argv[0], bozo_argv);	/* should not return */
+    execv(argv[0], argv);	/* should not return */
     _exit(1);
 #endif /* AFS_NT40_ENV */
 }
@@ -169,52 +202,32 @@ CreateDirs(const char *coredir)
 	(!strncmp
 	 (AFSDIR_USR_DIRPATH, AFSDIR_SERVER_BIN_DIRPATH,
 	  strlen(AFSDIR_USR_DIRPATH)))) {
-	if (MakeDir(AFSDIR_USR_DIRPATH))
-	    return errno;
+	MakeDir(AFSDIR_USR_DIRPATH);
     }
     if (!strncmp
 	(AFSDIR_SERVER_AFS_DIRPATH, AFSDIR_SERVER_BIN_DIRPATH,
 	 strlen(AFSDIR_SERVER_AFS_DIRPATH))) {
-	if (MakeDir(AFSDIR_SERVER_AFS_DIRPATH))
-	    return errno;
+	MakeDir(AFSDIR_SERVER_AFS_DIRPATH);
     }
-    if (MakeDir(AFSDIR_SERVER_BIN_DIRPATH))
-	return errno;
-    if (MakeDir(AFSDIR_SERVER_ETC_DIRPATH))
-	return errno;
-    if (MakeDir(AFSDIR_SERVER_LOCAL_DIRPATH))
-	return errno;
-    if (MakeDir(AFSDIR_SERVER_DB_DIRPATH))
-	return errno;
-    if (MakeDir(AFSDIR_SERVER_LOGS_DIRPATH))
-	return errno;
+    MakeDir(AFSDIR_SERVER_BIN_DIRPATH);
+    MakeDir(AFSDIR_SERVER_ETC_DIRPATH);
+    MakeDir(AFSDIR_SERVER_LOCAL_DIRPATH);
+    MakeDir(AFSDIR_SERVER_DB_DIRPATH);
+    MakeDir(AFSDIR_SERVER_LOGS_DIRPATH);
 #ifndef AFS_NT40_ENV
     if (!strncmp
 	(AFSDIR_CLIENT_VICE_DIRPATH, AFSDIR_CLIENT_ETC_DIRPATH,
 	 strlen(AFSDIR_CLIENT_VICE_DIRPATH))) {
-	if (MakeDir(AFSDIR_CLIENT_VICE_DIRPATH))
-	    return errno;
+	MakeDir(AFSDIR_CLIENT_VICE_DIRPATH);
     }
-    if (MakeDir(AFSDIR_CLIENT_ETC_DIRPATH))
-	return errno;
+    MakeDir(AFSDIR_CLIENT_ETC_DIRPATH);
 
-    if (symlink(AFSDIR_SERVER_THISCELL_FILEPATH,
-	    AFSDIR_CLIENT_THISCELL_FILEPATH)) {
-	if (errno != EEXIST) {
-	    return errno;
-	}
-    }
-    if (symlink(AFSDIR_SERVER_CELLSERVDB_FILEPATH,
-	    AFSDIR_CLIENT_CELLSERVDB_FILEPATH)) {
-	if (errno != EEXIST) {
-	    return errno;
-	}
-    }
+    symlink(AFSDIR_SERVER_THISCELL_FILEPATH, AFSDIR_CLIENT_THISCELL_FILEPATH);
+    symlink(AFSDIR_SERVER_CELLSERVDB_FILEPATH,
+	    AFSDIR_CLIENT_CELLSERVDB_FILEPATH);
 #endif /* AFS_NT40_ENV */
-    if (coredir) {
-	if (MakeDir(coredir))
-	    return errno;
-    }
+    if (coredir)
+	MakeDir(coredir);
     return 0;
 }
 
@@ -310,16 +323,15 @@ ReadBozoFile(char *aname)
 
     for (code = 0; code < MAXPARMS; code++)
 	parms[code] = NULL;
-    instp = typep = notifier = NULL;
     tfile = (FILE *) 0;
     if (!aname)
 	aname = (char *)bozo_fileName;
     tfile = fopen(aname, "r");
     if (!tfile)
 	return 0;		/* -1 */
-    instp = (char *)malloc(BOZO_BSSIZE);
-    typep = (char *)malloc(BOZO_BSSIZE);
-    notifier = notp = (char *)malloc(BOZO_BSSIZE);
+    instp = malloc(BOZO_BSSIZE);
+    typep = malloc(BOZO_BSSIZE);
+    notifier = notp = malloc(BOZO_BSSIZE);
     while (1) {
 	/* ok, read lines giving parms and such from the file */
 	tp = fgets(tbuffer, sizeof(tbuffer), tfile);
@@ -581,6 +593,122 @@ tweak_config(void)
 }
 #endif
 
+#if 0
+/*
+ * This routine causes the calling process to go into the background and
+ * to lose its controlling tty.
+ *
+ * It does not close or otherwise alter the standard file descriptors.
+ *
+ * It writes warning messages to the standard error output if certain
+ * fundamental errors occur.
+ *
+ * This routine has been tested on:
+ *
+ * AIX 4.2
+ * Digital Unix 4.0D
+ * HP-UX 11.0
+ * IRIX 6.5
+ * Linux 2.1.125
+ * Solaris 2.5
+ * Solaris 2.6
+ */
+
+#ifndef AFS_NT40_ENV
+static void
+background(void)
+{
+    /*
+     * A process is a process group leader if its process ID
+     * (getpid()) and its process group ID (getpgrp()) are the same.
+     */
+
+    /*
+     * To create a new session (and thereby lose our controlling
+     * terminal) we cannot be a process group leader.
+     *
+     * To guarantee we are not a process group leader, we fork and
+     * let the parent process exit.
+     */
+
+    if (getpid() == getpgrp()) {
+	pid_t pid;
+	pid = fork();
+	switch (pid) {
+	case -1:
+	    abort();		/* leave footprints */
+	    break;
+	case 0:		/* child */
+	    break;
+	default:		/* parent */
+	    exit(0);
+	    break;
+	}
+    }
+
+    /*
+     * By here, we are not a process group leader, so we can make a
+     * new session and become the session leader.
+     */
+
+    {
+	pid_t sid = setsid();
+
+	if (sid == -1) {
+	    static char err[] = "bosserver: WARNING: setsid() failed\n";
+	    write(STDERR_FILENO, err, sizeof err - 1);
+	}
+    }
+
+    /*
+     * Once we create a new session, the current process is a
+     * session leader without a controlling tty.
+     *
+     * On some systems, the first tty device the session leader
+     * opens automatically becomes the controlling tty for the
+     * session.
+     *
+     * So, to guarantee we do not acquire a controlling tty, we fork
+     * and let the parent process exit.  The child process is not a
+     * session leader, and so it will not acquire a controlling tty
+     * even if it should happen to open a tty device.
+     */
+
+    if (getpid() == getpgrp()) {
+	pid_t pid;
+	pid = fork();
+	switch (pid) {
+	case -1:
+	    abort();		/* leave footprints */
+	    break;
+	case 0:		/* child */
+	    break;
+	default:		/* parent */
+	    exit(0);
+	    break;
+	}
+    }
+
+    /*
+     * check that we no longer have a controlling tty
+     */
+
+    {
+	int fd;
+
+	fd = open("/dev/tty", O_RDONLY);
+
+	if (fd >= 0) {
+	    static char err[] =
+		"bosserver: WARNING: /dev/tty still attached\n";
+	    close(fd);
+	    write(STDERR_FILENO, err, sizeof err - 1);
+	}
+    }
+}
+#endif /* ! AFS_NT40_ENV */
+#endif
+
 static char *
 make_pid_filename(char *ainst, char *aname)
 {
@@ -678,17 +806,16 @@ bozo_CreateRxBindFile(afs_uint32 host)
     char buffer[16];
     FILE *fp;
 
+    if (host == htonl(INADDR_ANY)) {
+	host = htonl(0x7f000001);
+    }
+
     afs_inet_ntoa_r(host, buffer);
     bozo_Log("Listening on %s:%d\n", buffer, AFSCONF_NANNYPORT);
     if ((fp = fopen(AFSDIR_SERVER_BOZRXBIND_FILEPATH, "w")) == NULL) {
 	bozo_Log("Unable to open rxbind address file: %s, code=%d\n",
 		 AFSDIR_SERVER_BOZRXBIND_FILEPATH, errno);
     } else {
-	/* If listening on any interface, write the loopback interface
-	   to the rxbind file to give local scripts a usable addresss. */
-	if (host == htonl(INADDR_ANY)) {
-	    afs_inet_ntoa_r(htonl(0x7f000001), buffer);
-	}
 	fprintf(fp, "%s\n", buffer);
 	fclose(fp);
     }
@@ -775,23 +902,8 @@ main(int argc, char **argv, char **envp)
     }
 #endif
 
-#ifndef AFS_NT40_ENV
-    /* save args for restart */
-    bozo_argc = argc;
-    bozo_argv = malloc((argc+1) * sizeof(char*));
-    if (!bozo_argv) {
-	fprintf(stderr, "%s: Failed to allocate argument list.\n", argv[0]);
-	exit(1);
-    }
-    bozo_argv[0] = (char*)AFSDIR_SERVER_BOSVR_FILEPATH; /* expected path */
-    bozo_argv[bozo_argc] = NULL; /* null terminate list */
-#endif	/* AFS_NT40_ENV */
-
     /* parse cmd line */
     for (code = 1; code < argc; code++) {
-#ifndef AFS_NT40_ENV
-	bozo_argv[code] = argv[code];
-#endif	/* AFS_NT40_ENV */
 	if (strcmp(argv[code], "-noauth") == 0) {
 	    /* set noauth flag */
 	    noAuth = 1;
@@ -835,13 +947,6 @@ main(int argc, char **argv, char **envp)
 		exit(1);
 	    }
 	    rxMaxMTU = atoi(argv[++code]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
-		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
-			rxMaxMTU, RX_MIN_PACKET_SIZE,
-			RX_MAX_PACKET_DATA_SIZE);
-		exit(1);
-	    }
 	}
 	else if (strcmp(argv[code], "-auditlog") == 0) {
 	    auditFileName = argv[++code];
@@ -856,7 +961,7 @@ main(int argc, char **argv, char **envp)
 	} else if (strncmp(argv[code], "-pidfiles=", 10) == 0) {
 	    DoPidFiles = (argv[code]+10);
 	} else if (strncmp(argv[code], "-pidfiles", 9) == 0) {
-	    DoPidFiles = AFSDIR_LOCAL_DIR;
+	    DoPidFiles = AFSDIR_BOSCONFIG_DIR;
 	}
 	else {
 
@@ -912,33 +1017,26 @@ main(int argc, char **argv, char **envp)
     bnode_Register("cron", &cronbnode_ops, 2);
 
     /* create useful dirs */
-    i = CreateDirs(DoCore);
-    if (i) {
-	printf("bosserver: could not set up directories, code %d\n", i);
-	exit(1);
-    }
+    CreateDirs(DoCore);
 
     /* chdir to AFS log directory */
     if (DoCore)
-	i = chdir(DoCore);
+	chdir(DoCore);
     else
-	i = chdir(AFSDIR_SERVER_LOGS_DIRPATH);
+	chdir(AFSDIR_SERVER_LOGS_DIRPATH);
 
-    if (i) {
-	printf("bosserver: could not change to %s, code %d\n",
-	       DoCore ? DoCore : AFSDIR_SERVER_LOGS_DIRPATH, errno);
-	exit(1);
-    }
+#if 0
+    fputs(AFS_GOVERNMENT_MESSAGE, stdout);
+    fflush(stdout);
+#endif
 
     /* go into the background and remove our controlling tty, close open
        file desriptors
      */
 
 #ifndef AFS_NT40_ENV
-    if (!nofork) {
-	if (daemon(1, 0))
-	    printf("bosserver: warning - daemon() returned code %d\n", errno);
-    }
+    if (!nofork)
+	daemon(1, 0);
 #endif /* ! AFS_NT40_ENV */
 
     if ((!DoSyslog)
@@ -1014,20 +1112,9 @@ main(int argc, char **argv, char **envp)
 	exit(code);
     }
 
-    /* Disable jumbograms */
-    rx_SetNoJumbo();
-
-    if (rxMaxMTU != -1) {
-	rx_SetMaxMTU(rxMaxMTU);
-    }
-
     code = LWP_CreateProcess(BozoDaemon, BOZO_LWP_STACKSIZE, /* priority */ 1,
 			     /* param */ NULL , "bozo-the-clown",
 			     &bozo_pid);
-    if (code) {
-	bozo_Log("Failed to create daemon thread\n");
-        exit(1);
-    }
 
     /* try to read the key from the config file */
     tdir = afsconf_Open(AFSDIR_SERVER_ETC_DIRPATH);
@@ -1081,12 +1168,22 @@ main(int argc, char **argv, char **envp)
     rx_SetRxStatUserOk(bozo_rxstat_userok);
 
     afsconf_SetNoAuthFlag(tdir, noAuth);
-    afsconf_BuildServerSecurityObjects(tdir, 0, &securityClasses, &numClasses);
+    afsconf_BuildServerSecurityObjects(tdir, &securityClasses, &numClasses);
 
     if (DoPidFiles) {
 	bozo_CreatePidFile("bosserver", NULL, getpid());
     }
 
+    /* Disable jumbograms */
+    rx_SetNoJumbo();
+
+    if (rxMaxMTU != -1) {
+	if (rx_SetMaxMTU(rxMaxMTU) != 0) {
+	    bozo_Log("bosserver: rxMaxMTU %d is invalid\n", rxMaxMTU);
+	    exit(1);
+	}
+    }
+
     tservice = rx_NewServiceHost(host, 0, /* service id */ 1,
 			         "bozo", securityClasses, numClasses,
 				 BOZO_ExecuteRequest);
@@ -1108,7 +1205,7 @@ main(int argc, char **argv, char **envp)
 }
 
 void
-bozo_Log(const char *format, ...)
+bozo_Log(char *format, ...)
 {
     char tdate[27];
     time_t myTime;
