@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -18,25 +18,15 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/bucoord/dsvs.c,v 1.10.6.3 2009/03/16 13:32:14 shadow Exp $");
+#include <roken.h>
 
-#include <sys/types.h>
 #include <afs/cmd.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <sys/param.h>
-#endif
 #include <afs/com_err.h>
-
 #include <afs/bubasics.h>
+
 #include "bc.h"
-#include "bucoord_prototypes.h"
-    
+#include "bucoord_internal.h"
+
 struct ubik_client *cstructp;	/*Ptr to Ubik client structure */
 
 static int FreeVolumeEntryList(struct bc_volumeEntry *aentry);
@@ -56,7 +46,7 @@ bc_GetPartitionID(char *aname, afs_int32 *aval)
 
     /*bc_GetPartitionID */
 
-    register char tc;
+    char tc;
     char ascii[3];
 
     /* special-case "anything" */
@@ -129,10 +119,10 @@ bc_ParseHost(char *aname, struct sockaddr_in *asockaddr)
 
 {				/*bc_ParseHost */
 
-    register struct hostent *th;	/*Host entry */
-    afs_int32 addr;		/*Converted address */
+    struct hostent *th;	/*Host entry */
+    afs_uint32 addr;		/*Converted address */
     afs_int32 b1, b2, b3, b4;	/*Byte-sized address chunks */
-    register afs_int32 code;	/*Return code from sscanf() */
+    afs_int32 code;	/*Return code from sscanf() */
     afs_int32 tmp1, tmp2;
 
     /*
@@ -194,7 +184,7 @@ int
 bc_CreateVolumeSet(struct bc_config *aconfig, char *avolName,
 		   afs_int32 aflags)
 {
-    register struct bc_volumeSet **tlast, *tset, *nset;
+    struct bc_volumeSet **tlast, *tset, *nset;
 
     if (bc_FindVolumeSet(aconfig, avolName))
 	return -1;		/* already exists */
@@ -219,7 +209,7 @@ bc_CreateVolumeSet(struct bc_config *aconfig, char *avolName,
 }
 
 static int
-FreeVolumeEntry(register struct bc_volumeEntry *aentry)
+FreeVolumeEntry(struct bc_volumeEntry *aentry)
 {
     free(aentry->name);
     free(aentry->serverName);
@@ -229,9 +219,9 @@ FreeVolumeEntry(register struct bc_volumeEntry *aentry)
 }
 
 static int
-FreeVolumeEntryList(register struct bc_volumeEntry *aentry)
+FreeVolumeEntryList(struct bc_volumeEntry *aentry)
 {
-    register struct bc_volumeEntry *tnext;
+    struct bc_volumeEntry *tnext;
 
     while (aentry) {
 	tnext = aentry->next;
@@ -255,7 +245,7 @@ int
 bc_DeleteVolumeSet(struct bc_config *aconfig, char *avolName,
 		   afs_int32 *flags)
 {
-    register struct bc_volumeSet **tlast, *tset;
+    struct bc_volumeSet **tlast, *tset;
 
     *flags = 0;
     tlast = &aconfig->vset;
@@ -276,9 +266,9 @@ int
 bc_DeleteVolumeItem(struct bc_config *aconfig, char *avolName,
 		    afs_int32 anumber)
 {
-    register afs_int32 i;
-    register struct bc_volumeSet *tset;
-    register struct bc_volumeEntry *tentry, **tlast;
+    afs_int32 i;
+    struct bc_volumeSet *tset;
+    struct bc_volumeEntry *tentry, **tlast;
 
     tset = bc_FindVolumeSet(aconfig, avolName);
     if (!tset)
@@ -298,12 +288,12 @@ bc_DeleteVolumeItem(struct bc_config *aconfig, char *avolName,
 }
 
 int
-bc_AddVolumeItem(struct bc_config *aconfig, char *avolName, char *ahost, 
+bc_AddVolumeItem(struct bc_config *aconfig, char *avolName, char *ahost,
 		 char *apart, char *avol)
 {
     struct bc_volumeSet *tset;
-    register struct bc_volumeEntry **tlast, *tentry;
-    register afs_int32 code;
+    struct bc_volumeEntry **tlast, *tentry;
+    afs_int32 code;
 
     tset = bc_FindVolumeSet(aconfig, avolName);
     if (!tset)
@@ -339,7 +329,7 @@ struct bc_volumeSet *
 bc_FindVolumeSet(struct bc_config *aconfig, char *aname)
 {				/*bc_FindVolumeSet */
 
-    register struct bc_volumeSet *tvs;
+    struct bc_volumeSet *tvs;
 
     for (tvs = aconfig->vset; tvs; tvs = tvs->next) {
 	if (!strcmp(tvs->name, aname))
@@ -363,11 +353,11 @@ bc_FindVolumeSet(struct bc_config *aconfig, char *aname)
  *	expType - absolute or relative
  */
 
-int 
-bc_CreateDumpSchedule(struct bc_config *aconfig, char *adumpName, 
+int
+bc_CreateDumpSchedule(struct bc_config *aconfig, char *adumpName,
 		      afs_int32 expDate, afs_int32 expType)
 {
-    register struct bc_dumpSchedule *tdump;
+    struct bc_dumpSchedule *tdump;
     struct bc_dumpSchedule *parent, *node;
     afs_int32 code;
 
@@ -405,11 +395,11 @@ bc_CreateDumpSchedule(struct bc_config *aconfig, char *adumpName,
  * to strange places), so we must call bc_ProcessDumpSchedule when we're done.
  */
 int
-bc_DeleteDumpScheduleAddr(struct bc_config *aconfig, 
+bc_DeleteDumpScheduleAddr(struct bc_config *aconfig,
                           struct bc_dumpSchedule *adumpAddr)
 {
-    register struct bc_dumpSchedule **tlast, *tdump;
-    register struct bc_dumpSchedule *tnext;
+    struct bc_dumpSchedule **tlast, *tdump;
+    struct bc_dumpSchedule *tnext;
 
     /* knock off all children first */
     for (tdump = adumpAddr->firstChild; tdump; tdump = tnext) {
@@ -444,7 +434,7 @@ bc_DeleteDumpScheduleAddr(struct bc_config *aconfig,
 struct bc_dumpSchedule *
 bc_FindDumpSchedule(struct bc_config *aconfig, char *aname)
 {
-    register struct bc_dumpSchedule *tds;
+    struct bc_dumpSchedule *tds;
     for (tds = aconfig->dsched; tds; tds = tds->next) {
 	if (!strcmp(tds->name, aname))
 	    return tds;
@@ -459,7 +449,7 @@ bc_FindDumpSchedule(struct bc_config *aconfig, char *aname)
 int
 bc_DeleteDumpSchedule(struct bc_config *aconfig, char *adumpName)
 {
-    register struct bc_dumpSchedule *tdump;
+    struct bc_dumpSchedule *tdump;
 
     /* does a linear search of the dump schedules in order to find
      * the one to delete
@@ -490,7 +480,7 @@ bc_DeleteDumpSchedule(struct bc_config *aconfig, char *adumpName)
 int
 bc_ProcessDumpSchedule(struct bc_config *aconfig)
 {
-    register struct bc_dumpSchedule *tds;
+    struct bc_dumpSchedule *tds;
     struct bc_dumpSchedule *parentptr, *nodeptr;
     int retval;
 
@@ -539,8 +529,8 @@ bc_ProcessDumpSchedule(struct bc_config *aconfig)
  */
 
 int
-FindDump(struct bc_config *aconfig, char *nodeString, 
-	 struct bc_dumpSchedule **parentptr, 
+FindDump(struct bc_config *aconfig, char *nodeString,
+	 struct bc_dumpSchedule **parentptr,
 	 struct bc_dumpSchedule **nodeptr)
 {
     struct bc_dumpSchedule *dsptr;
