@@ -1822,8 +1822,9 @@ afsclient_AFSServerGetBegin(const void *cellHandle, void **iterationIdP,
     int rc = 0;
     afs_status_t tst = 0;
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
-    afs_admin_iterator_p iter = malloc(sizeof(afs_admin_iterator_t));
-    server_get_p serv = calloc(1, sizeof(server_get_t));
+    afs_admin_iterator_p iter =
+	(afs_admin_iterator_p) malloc(sizeof(afs_admin_iterator_t));
+    server_get_p serv = (server_get_p) calloc(1, sizeof(server_get_t));
     server_get_p serv_cache = NULL;
     const char *cellName = NULL;
     void *database_iter;
@@ -2653,7 +2654,7 @@ afsclient_RXDebugOpen(const char *serverName, afs_stat_source_t type,
 	goto fail_afsclient_RXDebugOpen;
     }
 
-    handle = malloc(sizeof(rxdebugHandle_t));
+    handle = (rxdebugHandle_p) malloc(sizeof(rxdebugHandle_t));
     if (!handle) {
 	close(sock);
 	tst = ADMNOMEM;
@@ -2738,7 +2739,7 @@ afsclient_RXDebugOpenPort(const char *serverName, int serverPort,
 	goto fail_afsclient_RXDebugOpenPort;
     }
 
-    handle = malloc(sizeof(rxdebugHandle_t));
+    handle = (rxdebugHandle_p) malloc(sizeof(rxdebugHandle_t));
     if (!handle) {
 	close(sock);
 	tst = ADMNOMEM;
