@@ -9,24 +9,18 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
 #include <afs/stds.h>
-#include <ctype.h>
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif
-#include <stdio.h>
-#include <string.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
-#include <errno.h>
+
 #include "ptclient.h"
 #include "ptuser.h"
 #include "pterror.h"
@@ -292,13 +286,11 @@ pr_Initialize(IN afs_int32 secLevel, IN const char *confDir, IN char *cell)
      * to force use of the KeyFile.  secLevel == 0 implies -noauth was
      * specified. */
     if (secLevel == 2) {
-	secFlags = AFSCONF_SECOPTS_LOCALAUTH;
-	secFlags |= AFSCONF_SECOPTS_ALWAYSENCRYPT;
-	code = afsconf_PickClientSecObj(tdir, secFlags, &info, cell, &sc, &scIndex, NULL);
-	if (code) {
-	    afs_com_err(whoami, code, "(getting key from local KeyFile)\n");
-        }
-
+	/* If secLevel is two assume we're on a file server and use
+	 * ClientAuthSecure if possible. */
+	code = afsconf_ClientAuthSecure(tdir, &sc, &scIndex);
+	if (code)
+	    afs_com_err(whoami, code, "(calling client secure)\n");
     } else if (secLevel > 0) {
 	secFlags = 0;
 	if (secLevel > 1)
@@ -496,9 +488,6 @@ pr_NameToId(namelist *names, idlist *ids)
     for (i = 0; i < names->namelist_len; i++)
 	stolower(names->namelist_val[i]);
     code = ubik_PR_NameToID(pruclient, 0, names, ids);
-    if (code == 0 && ids->idlist_len != names->namelist_len) {
-	code = 267281L; /* PRINTERNAL */
-    }
     return code;
 }
 
@@ -623,8 +612,6 @@ pr_ListMembers(char *group, namelist *lnames)
     afs_int32 code;
     afs_int32 gid;
 
-    memset(lnames, 0, sizeof(namelist));
-
     code = pr_SNameToId(group, &gid);
     if (code)
 	return code;
